/* Converts a pair of alleles into an integer code   */
/* a==1st allele, b==2nd allele, c==tot # of alleles */

int code_dna (int a, int b, int c)
{
   int   temp;

   if (a > b)
   {
      temp = a;
      a = b;
      b = temp;
   }
   if ((a + b) == 0)
      return 0;
   if (a == 1)
      return b;
   else
      return ((a - 1) * (2 * c - a + 2) * .5 + b - a + 1);
}

/* Converts an integer coded genotype into a pair   */
/* of integer coded alleles.                        */

void uncode_dna (int g, int *a, int *b, int c)
{
   int   left, right, count, flag;

   if (g)
   {
      left = 1;
      right = c;
      count = 0;
      flag = 0;
      while (count <= c && flag == 0)
      {
         count++;
         if (left <= g && g <= right)
         {
            flag = 1;
            *a = count;                 /* the 'row' allele */
            if (count == 1)
               *b = g;                  /* the 'col' allele */
            else
               *b = g + count - 1 - ((count - 1) * (2 * c - count + 2) * .5);
         }
         left = right + 1;
         right = left + c - count - 1;
      }
   }
   else
   {
      *a = 0;
      *b = 0;
   }
}

/* Returns the segregation probability from parent 
   genos: (par1, par2) to offspring geno: (off)     */

double sege (int par1, int par2, int off, int k)
{
   int   a1, a2, a3, a4, a5, a6;
   double z1, z2, seg;

   uncode_dna (par1, &a1, &a2, k);
   uncode_dna (par2, &a3, &a4, k);
   uncode_dna (off, &a5, &a6, k);

   /* phase the offspring (a5|a6) */
   z1 = .5;
   z2 = .5;
   if (a5 != a1 && a5 != a2)
      z1 = 0;
   if (a6 != a3 && a6 != a4)
      z2 = 0;
   if (a5 == a1 && a5 == a2)
      z1 = 1;
   if (a6 == a3 && a6 == a4)
      z2 = 1;
   seg = z1 * z2;

   /* phase is unambiguous  */
   if (a5 == a6)
      return (seg);

   else
   {
      /* phase the offspring (a6|a5) */
      z1 = .5;
      z2 = .5;
      if (a6 != a1 && a6 != a2)
         z1 = 0;
      if (a5 != a3 && a5 != a4)
         z2 = 0;
      if (a6 == a1 && a6 == a2)
         z1 = 1;
      if (a5 == a3 && a5 == a4)
         z2 = 1;
      seg = seg + (z1 * z2);
   }
   return (seg);
}

/* Returns the penetrance of genotype g relative to */
/* marker phenotype y under error rate epsilon.     */

double pene (int g, int y, double eps, int k)
{
   k = k * (k + 1) / 2;
   if (y == 0)
      return 1;
   if (g == y)
      return (1 - eps);
   return (eps / (k - 1));
}

void prev_functions (int n1, int n2, int *seq, int *ptr)
{
   int   i, j, k, l;
   for (i = n1; i <= n2; ++i)
   {
      nprlik[i] = seq[ptr[0]];
      ptr[0]++;

      for (j = 1; j <= nprlik[i]; ++j)
      {
         iprlik[j][i] = seq[ptr[0]];
         ptr[0]++;
      }
      if (nprlik[i] == 0)
      {
         j = nstore;
         nprlik[i] = nindi[nstore];
         k = nprlik[i] + 1;
      }

      else
      {
         for (j = 2; j <= nstore; ++j)
         {
            if ((nindi[j] == nprlik[i]) && (ndex[j] != 0))
            {
               k = 1;
               while ((k <= nprlik[i]) && (iprlik[k][i] == namsto[j][k]))
                  ++k;
               if (k == nprlik[i] + 1)
                  break;
            }
         }
      }

      if (j < nstore + 1)               /* prev lhd i found at stored lhd j */
      {
         nstlik[i] = nstart[j];
         ndex[j] = 0;
         maxlik[i] = 0;
         for (l = 1; l <= nprlik[i]; ++l)
         {
            k = 1;
            while ((k <= ninvol) && (namsto[j][l] != invol[k]))
               ++k;
            if (k == ninvol + 1)
            {
               ++ninvol;
               invol[ninvol] = namsto[j][l];
            }
            iprlik[l][i] = k;
            if (k > maxlik[i])
               maxlik[i] = k;
         }
      }

      else                              /* prev lhd i not found in store */
      {
         if (nprlik[i] != 1)
         {
            printf ("\n Exited in prev lhd\n");
            exit (1);
         }
         l = iprlik[1][i];
         nstlik[i] = 0;
         k = 1;
         while ((k <= ninvol) && (l != invol[k]))
            ++k;
         if (k == ninvol + 1)
            ++ninvol;
         iprlik[1][i] = k;
         maxlik[i] = k;
      }
   }
}

void repack ()
{
   int   ntrans, newst;
   int   i, j, nb, nt;
   ntrans = 0;
   newst = 0;
   for (i = 1; i <= nstore; ++i)
   {
      if (ndex[i] == 0)
         ntrans += nstart[i + 1] - nstart[i];
      else
      {
         newst += 1;
         if (ntrans > 0)
         {
            nt = nstart[i + 1] - nstart[i];
            nb = nstart[i];
            for (j = 1; j <= nt; ++j)
               store[nb + j - ntrans] = store[nb + j];
            ndex[newst] = 1;
            nstart[newst + 1] = nb + nt - ntrans;
            nindi[newst] = nindi[i];
            for (j = 1; j <= nindi[i]; ++j)
               namsto[newst][j] = namsto[i][j];
         }
      }
   }
   nstore = newst;
}

int rescale (ff, nn)
     double *ff;
     int nn;
{
   int   j;
   double fm;

   fm = 0.0;
   for (j = 1; j <= nn; ++j)
      if (ff[j] > fm)
         fm = ff[j];
   if (fm == 0.0)
   {
      printf ("\n Exited in rescale\n");
      exit (1);
   }
   if (fm <= SMALL)
   {
      for (j = 1; j <= nn; ++j)
         ff[j] *= SCALE;
      return (1);
   }
   return (0);
}

void varlik (ff, nn)
     double *ff;
     int nn;
{
   int   nzero, j;
   double fm, fv;
   nzero = 0;
   fm = 0.0;
   fv = 0.0;
   for (j = 1; j <= nn; ++j)
   {
      if (ff[j] == 0.0)
         ++nzero;
      else
      {
         fm += ff[j];
         fv += ff[j] * ff[j];
      }
   }
   fm = fm / nn;
   fv = (fv / nn) - fm * fm;
}

/* Returns a double, which is the Pr(marker phenos) at 
   a locus. Depending on the value of rvsp_flag, the 
   genotypes at that locus may be sampled conditional 
   on the marker phenos.                                  */

double peel_rvsp (int rp_flg, double eps, int fam_numb, int *seq,
                  int ta, double *freqs, int ***pgif, int *sf)
{
   /*Quick and Dirty start */
   int   count_undo = 0, gap, overwrt;
   /* Quick and Dirty end   */

   int   off, cp_fou[NP], upp[NP][3], cluster[4][NST], upp_count = 0;
   int   jj, jjj, pivot, itru_geno[NP], par1, par2, pp, cend, rlen, ii;
   int   kk, kkk, ptr;
   double probs[PNG];
   int   i, j, k, t, iscale, nch, nam, nt;
   int   iphen[NP], ifou[NP];
   double flik[BIG];
   double sum, p1, p2, term, u;
   int   ig, nf, nb;

   ngenty = (ta * (ta + 1)) / 2;
   iscale = 0;

   for (i = 0; i < NP; ++i)
   {
      ifou[i] = 0;
      iphen[i] = 0;
      cp_fou[i] = 0;
      itru_geno[i] = 0;
   }

   npeph = pgif[1][fam_numb][0];
   for (j = 1; j <= npeph; ++j)
   {
      nam = pgif[3][fam_numb][j];
      iphen[nam] = pgif[2 - rp_flg][fam_numb][j];
      ifou[nam] = pgif[4][fam_numb][j];
      cp_fou[nam] = ifou[nam];
   }

   for (i = 0; i < NST; ++i)
   {
      for (j = 0; j < NI; ++j)
         namsto[i][j] = 0;
      ndex[i] = 0;
   }

   nstore = 1;
   nstart[1] = 0;
   nstart[2] = ngenty;
   namsto[1][1] = NP + 1;
   ndex[1] = 1;
   nindi[1] = 1;
   k = 0;
   for (i = 1; i <= ta; i++)
      for (j = i; j <= ta; j++)
      {
         k++;
         if (i == j)
            store[k] = freqs[i - 1] * freqs[i - 1];
         else
            store[k] = 2 * freqs[i - 1] * freqs[j - 1];
      }
   sum = 0.0;
   for (j = 1; j <= ngenty; ++j)
      sum += store[j];
   if ((sum > 1.00001) || (sum < 0.99999))
   {
      printf ("\n Exited in geno freqs summation\n");
      exit (1);
   }
   /* start of peeling loop */
   noutin = 1;
   ptr = 1;
   while (noutin > 0)
   {
      noutin = seq[ptr];
      ptr++;
      for (i = 1; i <= noutin; ++i)
      {
         invol[i] = seq[ptr];
         ptr++;
      }
      fflush (stdout);
      ntrip = 0;
      npar = 0;
      ninvol = noutin;
      nsib = seq[ptr];
      ptr++;
      nlik = seq[ptr];
      ptr++;
      iwrt = seq[ptr];
      ptr++;

      for (i = 1; i <= nsib; ++i)
      {
         im = seq[ptr];
         ptr++;
         ife = seq[ptr];
         ptr++;
         nch = seq[ptr];
         ptr++;

         /* Quick and Dirty start */
         if (noutin == 1)               /* cut set contains a single pivot */
         {
            upp_count++;
            upp[upp_count][1] = im;
            upp[upp_count][2] = ife;
         }
         if (noutin == 2)
            count_undo++;               /* cut set contains two indivs. */
         /* Quick and Dirty end */

         for (j = 1; j <= nch; ++j)
         {
            ich[j] = seq[ptr];
            ptr++;
         }
         ifoum = ifou[im];
         ifou[im] = 0;
         ifoufe = ifou[ife];
         ifou[ife] = 0;

         j = 1;
         while ((im != invol[j]) && (j <= ninvol))
            ++j;
         iim = j;
         if (j == ninvol + 1)
         {
            ++ninvol;
            invol[j] = im;
         }

         j = 1;
         while ((ife != invol[j]) && (j <= ninvol))
            ++j;
         iife = j;
         if (j == ninvol + 1)
         {
            ++ninvol;
            invol[j] = ife;
         }

         for (k = 1; k <= nch; ++k)
         {
            j = 1;
            while ((ich[k] != invol[j]) && (j <= ninvol))
               ++j;
            if (j == ninvol + 1)
            {
               ++ninvol;
               invol[j] = ich[k];
            }

            ++ntrip;
            itrip[1][ntrip] = iim;
            itrip[2][ntrip] = iife;
            itrip[3][ntrip] = j;
            mtrip[ntrip] = iim;

            if (iife > iim)
               mtrip[ntrip] = iife;
            if (j > mtrip[ntrip])
               mtrip[ntrip] = j;
         }

         /* founder frequencies for not previously considered founder parents */
         if (ifoufe == 1)
         {
            ++npar;
            nprlik[npar] = 1;
            iprlik[1][npar] = iife;
            maxlik[npar] = iife;
            nstlik[npar] = 0;
         }
         if (ifoum == 1)
         {
            ++npar;
            nprlik[npar] = 1;
            iprlik[1][npar] = iim;
            maxlik[npar] = iim;
            nstlik[npar] = 0;
         }
      }
      nlik += npar;

      prev_functions (npar + 1, nlik, seq, &ptr);
      /* Set up the actual peeling combinations */
      for (i = 1; i <= ninvol; ++i)
         igen[i] = 1;
      iplace = 1;
      iout = 1;
      pen[noutin + 1] = 1;
      nt = ngenty;
      for (i = 1; i < noutin; ++i)
         nt *= ngenty;
      for (i = 1; i <= nt; ++i)
         flik[i] = 0.0;

      if (iwrt >= 2)
         for (i = 1; i <= noutin; ++i)
            for (j = 1; j <= ngenty; ++j)
               fmarg[i][j] = 0.0;
      /* Start the actual peeling combinations */
      t = ninvol;
      while (t > 0)
      {
         p1 = 1.0;
         if (iplace <= noutin)
         {
            iout = 1;
            for (i = 1; i <= noutin; ++i)
               iout = ngenty * (iout - 1) + igen[noutin - i + 1];

            /* insert penetrances for individuals to be peeled;
               for individuals to be output, check penetrances non-zero */

            for (i = iplace; i <= noutin; ++i)
               if (pene (igen[i], iphen[invol[i]], eps, ta) == 0)
                  break;
            if (i < noutin + 1)
            {
               imposs = i;
               p1 = 0.0;
            }
            else
               iplace = noutin + 1;
         }

         if (p1 > 0.0)
         {                              /* output penetrances ok */
            for (i = iplace; (i <= ninvol) && (p1 > 0.0); ++i)
            {
               p1 = pene (igen[i], iphen[invol[i]], eps, ta);
               pen[i + 1] = pen[i] * p1;
            }
            imposs = i - 1;

            if (p1 > 0.0)
            {
               term = pen[ninvol + 1];
               /* insert transitions */
               p2 = 1.0;
               for (i = 1; (i <= ntrip) && (p2 > 0.0); ++i)
                  p2 *=
                     sege (igen[itrip[1][i]], igen[itrip[2][i]],
                           igen[itrip[3][i]], ta);
               if (p2 == 0.0)
                  imposs = mtrip[i - 1];

               if (p2 > 0.0)
               {
                  term = term * p2;
                  /* insert terms from prev R-fns */
                  for (i = 1; i <= nlik; ++i)
                  {
                     ig = 1;
                     for (j = nprlik[i]; j >= 1; --j)
                        ig = ngenty * (ig - 1) + igen[iprlik[j][i]];
                     term *= store[nstlik[i] + ig];
                     if (term == 0.0)
                     {
                        imposs = maxlik[i];
                        break;
                     }
                  }

                  if (term > 0.0)
                  {
                     flik[iout] += term;
                     if (iwrt >= 2)
                        for (j = 1; j <= noutin; ++j)
                           fmarg[j][igen[j]] += term;
                  }
               }
            }
         }
         /* find next genotype combination */
         i = imposs;
         while ((igen[i] == ngenty) && (i > 0))
            --i;
         if (i > 0)
         {
            igen[i] += 1;
            iplace = i;
            for (j = i + 1; j <= ninvol; ++j)
               igen[j] = 1;
         }
         else
            t = 0;
      }
      /* end of gene combinations if i==0 */
      /* this was the last peel if noutin ==0 */
      if (noutin > 0)
         /* otherwise */
      {
         /* rescale the R-function if necessary 
            ( and check non-zero)  */
         iscale += rescale (flik, nt);
         /* call REPACK if space exhausted */
         nf = nstart[nstore + 1] + nt;
         if ((nf > 2 * BIG) || (nstore >= NST - 2))
            repack ();

         /* store computed R-function in appropriate position */
         nf = nstart[nstore + 1] + nt;
         if ((nf > 2 * BIG) || (nstore >= NST - 2))
         {
            printf ("\n Exited because nstart too small\n");
            exit (1);
         }

         nstore += 1;
         nstart[nstore + 1] = nf;
         nb = nstart[nstore];
         for (j = 1; j <= nt; ++j)
            store[nb + j] = flik[j];
         ndex[nstore] = 1;
         nindi[nstore] = noutin;
         namsto[nstore][0] = ninvol;
         for (j = 1; j <= ninvol; j++)
            namsto[nstore][j] = invol[j];

         varlik (flik, nt);
      }
      /* finish of peeling operation */
   }
   /* finished with all peeling */
   *sf = iscale;
#if 0
   /* Quick and Dirty (1) start */
   if (count_undo > 0)                  /* must massage the stored rfs */
   {
      count_undo = count_undo / 2;
      gap = 2 * ngenty * ngenty;
      k = nstart[nstore] + 1 - (gap * count_undo + ngenty * (count_undo - 1));
      overwrt = nstore - 3 * count_undo + 1;
      for (i = 1; i <= count_undo; i++)
      {
         /* move namsto[.][.] info */
         t = namsto[overwrt + 2 * (i - 1)][0];
         namsto[overwrt][0] = (t + namsto[overwrt + 2 * i - 1][0] - 1);
         t = t - 2;
         for (j = t; j >= 1; j--)
            namsto[overwrt][j + 3] = namsto[overwrt + 2 * (i - 1)][j + 2];
         t = t + 4;
         for (j = 1; j <= (namsto[overwrt + 2 * i - 1][0] - 2); j++)
         {
            namsto[overwrt][t] = namsto[overwrt + 2 * i - 1][j + 2];
            t++;
         }
         for (j = 1; j <= 3; j++)
            namsto[overwrt][j] = namsto[overwrt + 2 * i][j];
         overwrt++;
         for (j = 1; j <= ngenty; j++)  /* relocating the compnts. of ea. rf */
         {
            store[k] = store[k + gap * i];
            k++;
         }
      }
      nstore = overwrt - 1;
   }
   /* Quick and Dirty (1) end */
#endif
   /* Quick and Dirty (2) start */
   if (count_undo > 0)                  /* must massage the stored rfs */
   {
      count_undo = count_undo / 3;
      gap = 3 * ngenty * ngenty;
      k = nstart[nstore] + 1 - (gap * count_undo + ngenty * (count_undo - 1));
      overwrt = nstore - 4 * count_undo + 1;
      for (i = 1; i <= count_undo; i++)
      {
         /* move namsto[.][.] info */
         t = namsto[overwrt + 3 * (i - 1)][0] - 2;
         namsto[overwrt][0] = namsto[overwrt + 3 * i - 2][0] - 2;
         namsto[overwrt][0] += (t + namsto[overwrt + 3 * i - 1][0] + 1);
         for (j = t; j >= 1; j--)       /* 1st par-peel */
            namsto[overwrt][j + 3] = namsto[overwrt + 3 * (i - 1)][j + 2];
         t = t + 4;
         for (j = 1; j <= (namsto[overwrt + 3 * i - 2][0] - 2); j++)
         {
            /* 2nd par-peel */
            namsto[overwrt][t] = namsto[overwrt + 3 * i - 2][j + 2];
            t++;
         }
         for (j = 1; j <= (namsto[overwrt + 3 * i - 1][0] - 2); j++)
         {
            /* 3rd par-peel */
            namsto[overwrt][t] = namsto[overwrt + 3 * i - 1][j + 2];
            t++;
         }
         for (j = 1; j <= 3; j++)       /* incorporate single pivot info */
            namsto[overwrt][j] = namsto[overwrt + 3 * i][j];
         overwrt++;
         for (j = 1; j <= ngenty; j++)  /* relocating the compnts. of ea. rf */
         {
            store[k] = store[k + gap * i];
            k++;
         }
      }
      nstore = overwrt - 1;
   }
   /* Quick and Dirty (2) end */

   rlen = nstore;

   if (!rp_flg)
   {
      /* printf("Likelihood %e, scaled %d times \n",flik[1],iscale); */
      return (flik[1]);
   }

  /**************** Otherwise, Begin Reverse Peeling ************************/

   for (nb = 1; nb <= CHAIN; nb++)
   {
      u = rans ();                      /* back sample the last pivot, first */
      k = (nstore - 1) * ngenty;
      pivot = namsto[nstore][1];
      i = 1;
      probs[0] = (store[k + i] * pene (i, iphen[pivot], eps, ta)) / flik[1];
      while (u >= probs[0] && probs[0] < 1)
      {
         i++;
         probs[0] += (store[k + i] * pene (i, iphen[pivot], eps, ta)) / flik[1];
      }
      itru_geno[pivot] = i;

      t = 0;
      while (nstore > 1)                /* traverse the peeling sequence in reverse order */
      {
         if (namsto[nstore][0] > t)
            t = namsto[nstore][0];
         for (i = 0; i < 4; i++)
            for (j = 0; j <= t; j++)
               cluster[i][j] = 0;       /* init cluster */
         cluster[0][0] = t;             /* set cluster size */
         pivot = namsto[nstore][1];     /* store pivot's id */

         for (i = 1; i <= cluster[0][0]; i++)
         {
            cluster[0][i] = namsto[nstore][i]; /* the id's of the indivs in the cluster */
            cluster[1][i] = cp_fou[cluster[0][i]]; /* fou. status for all in the cluster */
            j = nstore;
            while (j > 1 && cluster[0][i] != namsto[j][1])
               j--;                     /* last prev_rf (if any) */
            if (j > 1 && cluster[0][i] != pivot)
               cluster[2][i] = (j - 1) * ngenty;
            else
               cluster[2][i] = 0;
         }

         /* The parents of each cluster are either in positions 2 and 3, or 
            they are in positions 1 and 2.                                        */

         i = 0;
         par1 = 0;
         while (i <= upp_count)
         {
            i++;
            if (cluster[0][1] == upp[i][1] && cluster[0][2] == upp[i][2])
               par1 = 1;
            if (cluster[0][1] == upp[i][2] && cluster[0][2] == upp[i][1])
               par1 = 1;
            if (cluster[0][2] == upp[i][1] && cluster[0][3] == upp[i][2])
               par1 = 2;
            if (cluster[0][2] == upp[i][2] && cluster[0][3] == upp[i][1])
               par1 = 2;
         }
         par2 = par1 + 1;               /*  found unique parent pair   */
         cluster[3][par1] = 1;
         cluster[3][par2] = 1;

         pp = cluster[3][1];            /* pp=1 means that pivot is a parent 
                                           relative to this cluster            */
         if (pp == 0)
         {
            for (i = 0; i < 4; i++)
            {
               cluster[i][t + 1] = cluster[i][1];
               cluster[i][1] = cluster[i][3];
               cluster[i][3] = cluster[i][t + 1];
            }
         }
         /* begin back sampling by cluster */
         if (pp)                        /* pivot is a parent              */
         {
            cend = cluster[0][0];
            par1 = cluster[0][1];
            par2 = cluster[0][2];
            for (jj = 1; jj <= ngenty; jj++)
            {
               term = 1.0;
               j = itru_geno[par1];
               for (kk = 3; kk <= cend; kk++)
               {
                  p1 = 0;
                  off = cluster[0][kk];
                  kkk = cluster[2][kk];
                  if (kkk == 0)         /* off has no prev_rf */
                     for (k = 1; k <= ngenty; k++)
                        p1 +=
                           pene (k, iphen[off], eps, ta) * sege (j, jj, k, ta);
                  else                  /* off has prev_rf */
                     for (k = 1; k <= ngenty; k++)
                        p1 +=
                           store[kkk + k] * pene (k, iphen[off], eps,
                                                  ta) * sege (j, jj, k, ta);
                  term *= p1;
                  if (term == 0)
                     break;
               }
               if (term != 0)
               {
                  jjj = cluster[2][2];
                  if (cluster[1][2] == 1 && jjj == 0) /* if founder and has no prev_rf */
                     term *= (pene (jj, iphen[par2], eps, ta) * store[jj]);
                  else
                     term *=
                        (pene (jj, iphen[par2], eps, ta) * store[jjj + jj]);
               }
               probs[jj] = term;
            }

            u = rans ();
            sum = 0;                    /* back sample the other parent */
            for (k = 1; k <= ngenty; k++)
               sum += probs[k];
            k = 1;
            probs[0] = probs[1] / sum;
            while (u >= probs[0] && probs[0] < 1)
            {
               k++;
               probs[0] += (probs[k] / sum);
            }
            itru_geno[par2] = k;

            jj = itru_geno[par2];       /* back sample kids */
            for (kk = 3; kk <= cend; kk++)
            {
               off = cluster[0][kk];
               kkk = cluster[2][kk];
               if (kkk != 0)
                  for (k = 1; k <= ngenty; k++)
                     probs[k] =
                        store[kkk + k] * pene (k, iphen[off], eps,
                                               ta) * sege (j, jj, k, ta);
               else
                  for (k = 1; k <= ngenty; k++)
                     probs[k] =
                        pene (k, iphen[off], eps, ta) * sege (j, jj, k, ta);
               sum = 0;
               for (i = 1; i <= ngenty; i++)
                  sum += probs[i];
               u = rans ();
               i = 1;
               probs[0] = probs[i] / sum;
               while (u >= probs[0] && probs[0] < 1)
               {
                  i++;
                  probs[0] += (probs[i] / sum);
               }
               itru_geno[off] = i;
            }
         }
         else                           /* if pivot is not a parent  */
         {
            j = 0;
            jjj = itru_geno[pivot];
            cend = cluster[0][0];
            par1 = cluster[0][1];
            par2 = cluster[0][2];
            for (i = 1; i <= (ngenty * ngenty); i++) /* big loop on probs */
            {
               if (((i - 1) % ngenty) == 0)
               {
                  j++;
                  jj = 0;
               }
               jj++;
               term = 1.0;
               for (kk = 4; kk <= cend; kk++) /* cycle thru the kids */
               {
                  p1 = 0;
                  off = cluster[0][kk];
                  kkk = cluster[2][kk];
                  if (kkk == 0)         /* no prev_rf */
                     for (k = 1; k <= ngenty; k++)
                        p1 +=
                           (pene (k, iphen[off], eps, ta) *
                            sege (j, jj, k, ta));
                  else                  /* prev_rf */
                     for (k = 1; k <= ngenty; k++)
                        p1 +=
                           (store[kkk + k] * pene (k, iphen[off], eps, ta) *
                            sege (j, jj, k, ta));
                  term *= p1;
                  if (term == 0)
                     break;             /* stop cycling thru kids */
               }
               if (term != 0)           /* cycle thru parents */
               {
                  term *= sege (j, jj, jjj, ta);
                  k = j;
                  for (kk = 1; kk <= 2; kk++)
                  {
                     ii = cluster[0][kk];
                     kkk = cluster[2][kk];
                     if (cluster[1][kk] == 1 && kkk == 0) /* if founder and no prev-rf */
                        term *= (store[k] * pene (k, iphen[ii], eps, ta));
                     else               /* nonfounder with prev_rf */
                        term *= (store[kkk + k] * pene (k, iphen[ii], eps, ta));
                     k = jj;
                  }
               }
               probs[i] = term;
            }                           /* done with conditional probs for parent pair */

            sum = 0;
            for (i = 1; i <= (ngenty * ngenty); i++)
               sum += probs[i];
            u = rans ();
            i = 1;
            probs[0] = probs[i] / sum;
            while (u >= probs[0] && probs[0] < 1)
            {
               i++;
               probs[0] += (probs[i] / sum);
            }
            if ((i % ngenty) != 0)
               itru_geno[par2] = i % ngenty;
            else
               itru_geno[par2] = ngenty;
            itru_geno[par1] = ((i - itru_geno[par2]) / ngenty + 1); /* parents sampled */

            j = itru_geno[par1];
            jj = itru_geno[par2];       /* back sample kids */
            for (kk = 4; kk <= cend; kk++)
            {
               off = cluster[0][kk];
               kkk = cluster[2][kk];
               if (kkk != 0)
                  for (k = 1; k <= ngenty; k++)
                     probs[k] =
                        store[kkk + k] * pene (k, iphen[off], eps,
                                               ta) * sege (j, jj, k, ta);
               else
                  for (k = 1; k <= ngenty; k++)
                     probs[k] =
                        pene (k, iphen[off], eps, ta) * sege (j, jj, k, ta);
               sum = 0;
               for (i = 1; i <= ngenty; i++)
                  sum += probs[i];
               u = rans ();
               i = 1;
               probs[0] = probs[i] / sum;
               while (u >= probs[0] && probs[0] < 1)
               {
                  i++;
                  probs[0] += (probs[i] / sum);
               }
               itru_geno[off] = i;
            }
         }
         nstore--;
      }

      for (i = 1; i <= npeph; i++)
      {
         j = pgif[3][fam_numb][i];
         if (pgif[1][fam_numb][i])
            pgif[2][fam_numb][i] = itru_geno[j];
      }

      nstore = rlen;
   }                                    /* resets nstore for another full round of reverse peeling  */
   return (1.0);
}

/* Notes: A cluster is a nuclear subfamily that is connected to the rest of the
          family through a single pivot. The reverse-peeling portion of this 
          progam assumes that any zero-looped family can be decomposed into 
          a series of conditionally independent clusters.
 
          The variable 'cluster' is a matrix of dimension (4 x 'cluster_size').
          The columns of 'cluster' are one-indexed, the rows are not.

          0th-row contains cluster member ids
          1st-row contains the founder status of ea. member
          2nd-row contains the positions of stored rfs, and is 0 when no rf exists
          3rd-row contains an indicator of which members are the parents

          The penetrance terms of individuals who are pivots, ARE NOT included in 
          any R-function. These penetrances are incorporated into the likelihood 
          calculation when the pivot becomes a "trivial" pivot (i.e. his/her upper 
          and lower parts have all been peeled).

          If a member of the cluster is a founder, his/her genotype frequencies 
          ARE included in the R-function corresponding to the pivot of that 
          cluster. At this time, his/her founder status is toggled. This is why, 
          reverse-peeling requires a copy of the original vector of contianing 
          the founder status of all individuals.                 

          Except in the case where the final individual in a peeling sequence has 
          penetrance values 1, the terms that are summed to produce the overall 
          likelihood are not stored anywhere!                                       */
