/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*****************************************************************************
  program name: gl_auto.c  (running as a version of lm_auto).
******************************************************************************/

/**
 * @defgroup lm_auto lm_auto
 * @ingroup autozyg
 * @{
 */

#define PNAM  "lm_auto"

#include "autozyg.h"

#define AUTO_CKPNTS 1000

void  gl_do_auto_sweep (int, int);
void  prnt_xxx (FILE *, int **, int, int);
void  prnt_fgl (FILE *, int **, int, int);
void  score_counts (int, int);

static FILE *oScorF = NULL;

static MSD_PTR p_msd = NULL;

static int nsw;
static int mc_itno;

                                        /* Counts and log-probabilities.      */
static double *ctot;                    /* CDLL by component.                 */
static double **prob;
static int tot_count;

                                      /* fgl_dump variables                */
static int ixxfgl;                      /* indicator of xxx or fgl output    */
static Indlist *pind;                   /* pointer to component indlist */

int main (int argc, char *argv[])
{
   int   nCompnt1, cmp;                 /* cmp indicators for sampling by cmpt */
   int   mul;                           /* mul is indicator of MM-sampling.   */
   int   c, i, s, t;

   FILE *oSeedF, *iXtra;

   nsw = setupAutozyg (argc, argv, PNAM, 0);

   if (DebugMain)
   {
      list_switches (nsw);
      list_nghds ();
      printf ("\n Done with lists\n");
      fflush (stdout);
   }

   setupSampling ();
   prob = setupInherit (McmcMarkersOnly);

/* Set up flags for sampling etc.                                             */
   if (!(iXtra = open_fil (XTRA)))
      give_up (ERR_qit);
   fscanf (iXtra, "%d %d %d", &mul, &MaxNMexact, &ixxfgl);

/* Set up for MM-sampling: from lm_multiple.                                  */
   if (mul == 1)
   {
      Max_update = 8;
      P_meio = 0.2;
      P_ind = 0.3;
      P_sib = 0.3;
      P_3gen = 0.2;

      setup_marriage ();
      prnt_marriage ();
      printf ("\n Done with marriages\n\n");
      fflush (stdout);
   }

   nCompnt1 = 0;
   if (!GlobalMcmc)
   {
      /* Number of components to be iid sampled.                              */
      for (c = 1; c <= nCompnt; c++)
         if (cmplist[c].nmeioses <= MaxNMexact)
            nCompnt1++;
   }

   if (PrintLevel >= 4)
   {
      ctot = malloc (n_compnts * sizeof (double));
      alloc_chk (WHERE, ctot);
   }

   /* Open output scores file, if requested to do so.                         */

   if (FilIt && !oScorF)
      if (!(oScorF = open_fil (OSCORE)))
         give_up (ERR_qit);

   prntStatusBMC ();
   fflush (stdout);

   if (GlobalMcmc)
   {
      cmp = 0;
      if (PrintLevel >= 4)
      {
         if (p_msd == NULL)
            p_msd = alloc_msd ();
         tot_count = 0;

         alloc_logprob ();
      }

      setup_sampling_cmp (cmp, mul, NMarks);

      /* Now do burn-in iterations.                                           */

      printf (" Now starting %d iterations of burn-in\n", NBrnIt);
      for (t = 1; t <= NBrnIt; t++)
      {
         if (!(t % ChkIt))
         {
            printf (" Starting burn-in iteration %d\n", t);
            fflush (stdout);
         }
         sampling_cmp (t, cmp, McmcMarkersOnly, mul, NMarks, nFGL, prob);
      }

      /* Now do MCMC sampling, and score results.                             */

      printf (" Now starting %d main MCMC iterations\n", NIt);
      fflush (stdout);

      for (mc_itno = 0; mc_itno < NIt; ++mc_itno)
         gl_do_auto_sweep (mul, cmp);   /* Do main mcmc iterations.           */

      printf ("\n");
      prntStatusFMC ();

      for (c = 0; c < n_compnts; ++c)   /* Now print results.                 */
      {
         pr_cmpnthead (c);
         if (PrintLevel >= 4)
         {
            pr_log_prob_seg (1, NMarks, -cmplist[c + 1].nmeioses,
                             p_msd->tot_log_prob_seg[c], tot_count);
            pr_log_prob_pen (1, NMarks, p_msd->tot_log_prob_pen[c], tot_count);
            pr_reco_counts (0, NMarks, p_msd->recounts[c]);
         }
      }

      if (PrintLevel >= 4)
      {
         free_msd (p_msd);
         free_logprob ();
      }

      free_sampling_cmp ();
   }

   else
   {
      printf (" Going to do stuff by components\n\n");
      fflush (stdout);

      for (cmp = 1; cmp <= n_compnts; ++cmp)
      {

         if (PrintLevel >= 4)
         {
            p_msd = alloc_msd ();
            tot_count = 0;

            alloc_logprob ();
         }

         if (cmplist[cmp].nmeioses <= MaxNMexact)
         {
            printf ("\n Component %d will be iid sampled ", cmp);
            printf ("and/or exact marginal computation will be done\n\n");
            fflush (stdout);

            /* fwd computation only */
            exact_smpl (1, 0, cmp, 0, NMarks, nFGL, prob, NMarks, ctot);
            /* iid sampling if reqd  */
            mc_itno = NIt / (ScorIt * FilIt);

            for (t = 0; t < mc_itno; ++t)
            {
               exact_smpl (0, 1, cmp, 0, NMarks, nFGL, prob, NMarks, ctot);

               if (PrintLevel >= 4)
                  score_counts (cmp, 0);

               /* TEMP CHECK EAT FIXME: print out meiosis indicators directly.

                  fprintf(oScorF, "\n");
                  for (j=0; j<=NMarks; ++j)
                  { pind = cmplist[cmp].people->next;
                  while (pind)
                  { i = pind->current->index;
                  if (!isNullNode(nghd[i].pa))
                  fprintf (oScorF, " %d %d",xx0[j][i],xx1[j][i]);
                  pind = pind -> next;
                  }
                  }
                  fflush(oScorF);
                */

               if (ixxfgl == 0)
               {
                  for (i = cmplist[cmp].meioses[0];
                       i <= cmplist[cmp].meioses[(cmplist[cmp].nmeioses - 1)];
                       ++i)
                  {
                     c = (meiosis[i].ind).index;
                     s = meiosis[i].sex;
                     if (s == 0)
                        prnt_xxx (oScorF, xx0, NMarks, c);
                     else
                        prnt_xxx (oScorF, xx1, NMarks, c);
                  }
               }
               else
               {
                  pind = cmplist[cmp].people->next;
                  while (pind)
                  {
                     i = pind->current->index;
                     {
                        prnt_fgl (oScorF, gg0, NMarks, i);
                        prnt_fgl (oScorF, gg1, NMarks, i);
                     }
                     pind = pind->next;
                  }
               }
            }
         }

         else
         {
            /* Sampling computation to be done separately by component.       */

            setup_sampling_cmp (cmp, mul, NMarks);

            /* Now do burn-in iterations.                                     */

            printf (" Now starting %d iterations of burn-in\n", NBrnIt);
            for (t = 1; t <= NBrnIt; t++)
            {
               if (!(t % ChkIt))
               {
                  printf (" Starting burn-in iteration %d\n", t);
                  fflush (stdout);
               }
               sampling_cmp (t, cmp, McmcMarkersOnly, mul, NMarks, nFGL, prob);
            }

            /* Now do MCMC sampling, and score results.                       */

            printf (" Now starting %d main MCMC iterations\n", NIt);
            fflush (stdout);

            for (mc_itno = 0; mc_itno < NIt; ++mc_itno)
               gl_do_auto_sweep (mul, cmp); /* do main mcmc iterations   */

            printf ("\n");
            prntStatusFMC ();
            free_sampling_cmp ();
         }

         /* Now print out the results.                                        */
         pr_cmpnthead (cmp - 1);
         c = 0;

         if (PrintLevel >= 4)
         {
            pr_log_prob_seg (1, NMarks, -cmplist[cmp].nmeioses,
                             p_msd->tot_log_prob_seg[c], tot_count);
            pr_log_prob_pen (1, NMarks, p_msd->tot_log_prob_pen[c], tot_count);
            pr_reco_counts (0, NMarks, p_msd->recounts[c]);

            free_msd (p_msd);
            free_logprob ();
         }
         fflush (stdout);
      }
   }

   /* ****  Now we are done.                                             **** */
   fflush (stdout);

   /* Free up allocated memory.                                               */
   free_xxgg ();

   if (PrintLevel >= 4)
      free_1d ((void *) &ctot);

   locus_peel_free ();
   free_allocate_loci ();
   free_chromo (0);
   free_gen_pen ();
   free_seg_pen (NMarks + 1, &prob);
   free_msmpl ();
   free_traits ();

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }
   if (FilIt)
      fclose (oScorF);

   if (Scan)
      free_perm ();

   exit (EXIT_SUCCESS);
}

void gl_do_auto_sweep (int amul, int acmp)
{
/* local variables: */
   int   c, s, i;

   /* now do the next MCMC sampling iteration */

   sampling_cmp (mc_itno + 1, acmp, McmcMarkersOnly, amul, NMarks, nFGL, prob);

   /* score results */

   if (!((mc_itno + 1) % ScorIt))
   {
      /* score tot log-liks & recombs for this component */
      if (!acmp)
      {
         if (PrintLevel >= 4)
         {
            c = 1;
            score_counts (acmp, c);
         }

         /* Test for output to scores file.                                   */
         if (FilIt && !((mc_itno + 1) % (ScorIt * FilIt)))
         {
            if (ixxfgl == 0)
            {
               for (i = 1; i <= nsw; ++i)
               {
                  c = (meiosis[i].ind).index;
                  s = meiosis[i].sex;
                  if (s == 0)
                     prnt_xxx (oScorF, xx0, NMarks, c);
                  else
                     prnt_xxx (oScorF, xx1, NMarks, c);
               }
            }
            else
            {
               for (i = 0; i < pedsize; ++i)
               {
                  prnt_fgl (oScorF, gg0, NMarks, i);
                  prnt_fgl (oScorF, gg1, NMarks, i);
               }
            }
         }
      }

      else
      {
         if (PrintLevel >= 4)
            score_counts (acmp, 0);

         /* Test for output to scores file.                                   */
         if (FilIt && !((mc_itno + 1) % (ScorIt * FilIt)))
         {
            if (ixxfgl == 0)
            {
               for (i = cmplist[acmp].meioses[0];
                    i <= cmplist[acmp].meioses[(cmplist[acmp].nmeioses - 1)];
                    ++i)
               {
                  c = (meiosis[i].ind).index;
                  s = meiosis[i].sex;
                  if (s == 0)
                     prnt_xxx (oScorF, xx0, NMarks, c);
                  else
                     prnt_xxx (oScorF, xx1, NMarks, c);
               }
            }
            else
            {
               pind = cmplist[acmp].people->next;
               while (pind)
               {
                  i = pind->current->index;
                  {
                     prnt_fgl (oScorF, gg0, NMarks, i);
                     prnt_fgl (oScorF, gg1, NMarks, i);
                  }
                  pind = pind->next;
               }
            }
         }
      }
   }

   return;
}

/* score_counts() added as separate routine: EAT Aug 2008 */

void score_counts (int scmp, int cc)
{
   int   c, j;

   if (!scmp)
   {
      for (c = 0; c < n_compnts; ++c)
      {
         ctot[c] = 0.0;
         full_seg_prob_cmp (NMarks, c, p_msd->recounts[c],
                            p_msd->temp_log_prob_seg[c]);
         for (j = 1; j < NMarks; ++j)
         {
            p_msd->tot_log_prob_pen[c][j] += log (prob[c][j]);
            p_msd->tot_log_prob_seg[c][j] += p_msd->temp_log_prob_seg[c][j];
            ctot[c] += (log (prob[c][j]) + p_msd->temp_log_prob_seg[c][j]);
         }
         p_msd->tot_log_prob_pen[c][NMarks] += log (prob[c][NMarks]);
         ctot[c] += log (prob[c][NMarks]);
      }
      ++tot_count;
   }

   else
   {
      ctot[cc] = 0.0;
      full_seg_prob_cmp (NMarks, scmp - 1, p_msd->recounts[cc],
                         p_msd->temp_log_prob_seg[cc]);
      for (j = 1; j < NMarks; ++j)
      {
         p_msd->tot_log_prob_pen[cc][j] += log (prob[scmp - 1][j]);
         p_msd->tot_log_prob_seg[cc][j] += p_msd->temp_log_prob_seg[cc][j];
         ctot[cc] += (log (prob[cc][j]) + p_msd->temp_log_prob_seg[cc][j]);
      }
      p_msd->tot_log_prob_pen[cc][NMarks] += log (prob[scmp - 1][NMarks]);
      ctot[cc] += log (prob[scmp - 1][NMarks]);
      ++tot_count;
   }

   return;
}

/* routine added EAT, April 2008  */

void prnt_xxx (FILE * fptr, int **xxx, int nmk, int ii)
{
   int   j, sw;

   sw = 0;
   for (j = 2; j <= nmk; ++j)
      if (xxx[j][ii] != xxx[j - 1][ii])
         ++sw;
   fprintf (fptr, "%d %d %d %d", ii, xxx[0][ii], xxx[1][ii], sw);
   for (j = 2; j <= nmk; ++j)
      if (xxx[j][ii] != xxx[j - 1][ii])
         fprintf (fptr, " %d", MarSel[j - 1]);
   fprintf (fptr, "\n");

   return;
}

/* routine added EAT, December 2008 */

void prnt_fgl (FILE * fptr, int **ggg, int nmk, int ii)
{
   int   j, sw;

   sw = 0;
   for (j = 2; j <= nmk; ++j)
      if (ggg[j][ii] != ggg[j - 1][ii])
         ++sw;
   fprintf (fptr, "%s %d %d %d", nghd[ii].self, ggg[0][ii], ggg[1][ii], sw);
   for (j = 2; j <= nmk; ++j)
      if (ggg[j][ii] != ggg[j - 1][ii])
         fprintf (fptr, " %d %d ", MarSel[j - 1], ggg[j][ii]);
   fprintf (fptr, "\n");

   return;
}
