/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include "nghds.h"

/*****************************************************************
  Function to read pedigree data in from a file written in free format and
  to form a data matrix for the pedigree.
  This version uses the "genetic data columns" as fixed-effects indicators,
  and also reads in a quantitative data value "quant" -- becomes "x"
  The nghds() and position() routines are then called

  Various checks on pedigree size etc are carried out. The global variables
  "pedsize" and "n_compnts" are also evaluated here.

  Modified out of input_data() in Quant library;  6/8/93. (EAT).

*****************************************************************/

void input_ped (FILE * fp, double **quant)
{
   fpos_t *fp_pos;
   int   i, j, k, n;
   KEYS_PP ppK;

   if (NReals && TraitTyp == QUANTITATIVE && !quant)
      give_up (ERR_qnt, "input_ped");

/* If needed, allocate storage to save the input stream's current position.   */

   if (SexIn != N)
      fp_pos = (fpos_t *) malloc (sizeof (fpos_t));

   for (i = 0; i < PedSize; ++i)
   {
      /* Skip the first three columns (the names are already in ppKeyTab) */
      for (j = 0; j < NNams; ++j)
         n = fscanf (fp, "%*s");
      if (n == EOF)
         break;

      data[i][0] = i;
      /* the ix in the keytable starts from 1 */
      data[i][1] = i + 1;               /* index of i + 1 */
      ppK = ppKeyTab + i;
      data[i][2] = (*ppK)->ixPa;        /* index of father */
      data[i][3] = (*ppK)->ixMa;        /* index of mother */

/* Unless the user explicitly excluded the presence of gender data by using   */
/* an "input record gender absent" statement, read this individual's sex into */
/* the fourth column of data [][].  Then reset the input stream's position to */
/* read in any selected integer data.                                         */

      if (SexIn != N)
      {
         n = fgetpos (fp, fp_pos);      /* Save the input stream position.    */
         n = fscanf (fp, "%d", &data[i][4]);
         n = fsetpos (fp, fp_pos);      /* Reset the input stream position.   */
      }

      for (j = 0; j < NInts; ++j)       /* Selected integers are read.        */
         if ((k = SelInt[j]))
            n = fscanf (fp, "%d", &data[i][k]);
         else
            n = fscanf (fp, "%*d");

      for (j = 0; j < NReals; ++j)      /* Selected reals are read.           */
         if ((k = SelReal[j]))
            n = fscanf (fp, "%lf", &quant[k - 1][i]);
         else
            n = fscanf (fp, "%*f");

      if (n == EOF)
         break;
   }

   if (PrintLevel >= 3)
   {
      fprintf (stdout, " Pedigree records contain:  names");
      if (NInts > 0)
         fprintf (stdout, ", %d integer%s", NInts, NInts > 1 ? "s" : "");
      if (NReals > 0)
         fprintf (stdout, ", %d real number%s", NReals, NReals > 1 ? "s" : "");
      fprintf (stdout, "\n\n");
      fflush (stdout);
   }

   pedsize = i;

   fscanf (fp, "%*s");                  /* FIX -- DO NOT DELETE -- EAT 6/2003 */

/* Free the storage used to save the input stream's current position.         */

   if (SexIn != N)
      free (fp_pos);

   return;
}
