/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*
 * proc_kin() has as its input the results of parsing the parameter
 * statements by proc_pars() for program "kin".  It checks this data
 * request, and if there are no errors, builds data structures used by
 * kin.
 *                                                         MAJ May 2002
 */

int proc_kin (void)
{
   int   k, c;

   if (!test_compu_kin ())              /* checks for compute statement(s)   */
      if (proc_err ())
         return FALSE;

   if (!test_reco_frq ())               /* sorts recomb freqs, if present    */
      if (proc_err ())
         return FALSE;

   if (n_err)
      return proc_end ();

   find_ncmps ();                       /* determines no. of components      */

   alloc_kin ();

   for (k = KINSHP; k <= INBRD2; ++k)   /* makes arrays of pedigee names for */
      for (c = 0; c < NKinCmp; ++c)     /* which coeffs are requested        */
         mak_compu_kin (k, c);

   if (first_2loc_node)
   {
      if (!NInbreedRF)                  /* if not supplied, uses default     */
         mak_def_reco ();               /* recomb freqs for 2-locus inbreed  */
      prnt_reco_frq ();
   }

   for (c = 0; c < NKinCmp; ++c)        /* prints component no., computation */
   {                                    /* type and names                    */
      prnt_kin_cmp (c);
      for (k = KINSHP; k <= INBRD2; ++k)
         prnt_compu_kin (k, c);
   }
   mak_kin_inpt ();                     /* set up controls for reading       */

   return TRUE;
}

int test_compu_kin (void)
{
   if (!first_kins_node && !first_inbreed_node && !first_2loc_node)
   {
      err_typ = ERK_KIN;
      return FALSE;
   }
   return TRUE;
}

int test_reco_frq (void)
{
   int   i, j;

   if (!NInbreedRF)
      return TRUE;

   if (NInbreedRF > 1)
      qsort (InbreedRF, NInbreedRF, sizeof (double), qfunc_doub);

   for (i = 0; i < NInbreedRF - 1;)     /* checks for duplicate frequencies  */
      if (InbreedRF[i] == InbreedRF[i + 1])
      {
         --NInbreedRF;
         for (j = i + 1; j < NInbreedRF - 1; ++j)
            InbreedRF[j] = InbreedRF[j + 1];

         err_typ = ERK_DRF;
      }
      else
         ++i;

   if (err_typ)
      return FALSE;

   else
      return TRUE;
}

void find_ncmps (void)
{
   int   i;
   KINS *p_k;

   for (i = 0; i < 3; ++i)              /* for ea of 3 kinds of compute       */
   {                                    /* requests                           */
      if (i == 0)
         p_k = first_kins_node;
      else if (i == 1)
         p_k = first_inbreed_node;
      else if (i == 2)
         p_k = first_2loc_node;

      if (p_k)                          /* finds the biggest component no.    */
      {
         while (p_k->p_next)
            p_k = p_k->p_next;
         if (NKinCmp < p_k->component)
            NKinCmp = p_k->component;
      }
   }
   return;
}

void alloc_kin (void)
{
   int   i;

   NKinNams = malloc (3 * sizeof (int *));
   alloc_chk (WHERE, NKinNams);
   KinNam = malloc (3 * sizeof (char ***));
   alloc_chk (WHERE, KinNam);
   KinIx = malloc (3 * sizeof (int *));
   alloc_chk (WHERE, KinIx);
   for (i = 0; i < 3; ++i)
   {
      NKinNams[i] = malloc (NKinCmp * sizeof (int));
      alloc_chk (WHERE, NKinNams[i]);
      KinNam[i] = malloc (NKinCmp * sizeof (char **));
      alloc_chk (WHERE, KinNam[i]);
      KinIx[i] = malloc (NKinCmp * sizeof (int *));
      alloc_chk (WHERE, KinIx[i]);
   }
   return;
}

void mak_compu_kin (int kc, int cmp)
{
   int   i;
   KINS *p_k;

   if (kc == KINSHP)                    /* gets start of list of requests of  */
      p_k = first_kins_node;            /* specified computation type         */
   else if (kc == INBRD)
      p_k = first_inbreed_node;
   else if (kc == INBRD2)
      p_k = first_2loc_node;

   if (p_k)                             /* checks for specified component     */
      while (p_k->p_next && p_k->component < cmp + 1)
         p_k = p_k->p_next;

   if (!p_k || p_k->component != cmp + 1)
   {
      NKinNams[kc][cmp] = 0;
      KinNam[kc][cmp] = NULL;
      KinIx[kc][cmp] = NULL;
   }
   else
   {
      NKinNams[kc][cmp] = p_k->no_nams;
      KinNam[kc][cmp] = malloc (p_k->no_nams * sizeof (char *));
      alloc_chk (WHERE, KinNam[kc][cmp]);
      KinIx[kc][cmp] = malloc (p_k->no_nams * sizeof (int));
      alloc_chk (WHERE, KinIx[kc][cmp]);

      KinNam[kc][cmp][0] = p_k->p_nams;
      for (i = 1; i < NKinNams[kc][cmp]; ++i)
         KinNam[kc][cmp][i] = KinNam[kc][cmp][i - 1] + MAXNAMLEN + 1;
   }
   return;
}

void mak_def_reco (void)
{
   int   i;

   NInbreedRF = ndefaultRF;

   InbreedRF = malloc (NInbreedRF * sizeof (double));
   alloc_chk (WHERE, InbreedRF);

   for (i = 0; i < NInbreedRF; ++i)
      InbreedRF[i] = defaultRF[i];

   printf ("\n Default values are used for recombination frequencies\n");

   return;
}

void prnt_kin_cmp (int cmp)
{
   if (NKinNams[0][cmp] || NKinNams[1][cmp] || NKinNams[2][cmp])
      printf ("\n Component %d:\n", cmp + 1);

   return;
}

void prnt_compu_kin (int kc, int cmp)
{
   int   i;

   char *msg[3] = {
      "\n    Kinship coefficients",
      "\n    Inbreeding coefficients",
      "\n    Two-locus inbreeding coefficients"
   };

   if (!(NKinNams[kc][cmp]))
      return;

   printf (msg[kc]);
   printf (" to be computed for:\n      ");

   for (i = 0; i < NKinNams[kc][cmp]; ++i)
   {
      printf (" %s", KinNam[kc][cmp][i]);
      if (kc == KINSHP && i % 2)
         printf (" ");
   }
   printf ("\n");

   return;
}

void prnt_reco_frq (void)
{
   int   i;

   printf ("\n Recombination frequencies for computing 2-locus inbreeding"
           " coefficients are:\n    ");
   for (i = 0; i < NInbreedRF; ++i)
      printf ("  %s", dub2nlz (InbreedRF[i], 6, 3));

   printf ("\n\n");

   return;
}

void mak_kin_inpt (void)
{
   alloc_sel ();

   SelInt[0] = 4;                       /* (gender)                          */

   /*                                    * NColData is the number of columns
    *                                    * to be allocated to the "data"
    *                                    * array for input_ped()             */
   NColData = 5;

   return;
}
