/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

int set_mc_iter (void)                  /* set MC iterations statement        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERR_SNI;
      NIt = j_par[0];
   }
   return err_typ;
}

int set_burn_iter (void)                /* set burn-in iterations             */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 0)
         err_typ = ERR_SBI;
      NBrnIt = j_par[0];
   }
   return err_typ;
}

int set_prel_iter (void)                /* set pseudo-prior iterations        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 0)
         err_typ = ERR_SPI;
      NPseudoPrIt = j_par[0];
   }
   return err_typ;
}

int set_lsmpl_prob (void)               /* set l-sampler probability          */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (d_par[0] < 0. || d_par[0] > 1.0)
         err_typ = ERA_BLS;

      LSamplProb = d_par[0];
   }
   return err_typ;
}

int set_pseudo_pr (void)                /* set pseudo-priors                  */
{
   if (chk_stat ())
   {
      if (chk_pseudo ())
      {
         if (!chk_repeat ())
            reset_set_pseudo ();
         set_set_pseudo ();
      }
   }
   return err_typ;
}

int set_batches_ve (void)               /* set I batches MC vari estimation   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 0)
         err_typ = ERR_BVE;
      NBtch = j_par[0];
   }
   return err_typ;
}

int set_tstpos_wind (void)              /* set test position window width     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 0 || d_par[0] > 100)
         err_typ = ERA_TPW;

      WindWdth = j_par[0];
   }
   return err_typ;
}

int set_trait_geno (void)               /* set trait data genotypic           */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PedDatTyp != UNDEF)
         err_typ = ERR_STT;
      PedDatTyp = GENOTYPIC;
      TraitTyp = GENOTYPIC;
   }
   return err_typ;
}

int set_trait_disc (void)               /* set trait data discrete            */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PedDatTyp != UNDEF)
         err_typ = ERR_STT;
      PedDatTyp = DISCRETE;
      TraitTyp = DISCRETE;
   }
   return err_typ;
}

int set_trait_liab (void)               /* set trait discrete w liab classes  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PedDatTyp != UNDEF)
         err_typ = ERR_STT;
      PedDatTyp = DISCR_LIAB;
      TraitTyp = DISCRETE;
   }
   return err_typ;
}

int set_trait_quant (void)              /* set trait data quantitative        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PedDatTyp != UNDEF)
         err_typ = ERR_STT;
      PedDatTyp = QUANTITATIVE;
      TraitTyp = QUANTITATIVE;
   }
   return err_typ;
}

int set_sqimp_prop (void)               /* set sequential imputatn proposals  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 0)
         err_typ = ERR_SIP;
      SeqImpIt = j_par[0];
   }
   return err_typ;
}

int sampl_by_scan (void)                /* sample by scan                     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (Scan == N)
         err_typ = ERR_SCN;
      Scan = Y;
   }
   return err_typ;
}

int sampl_by_step (void)                /* sample by step                     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (Scan == Y)
         err_typ = ERR_SCN;
      Scan = N;
   }
   return err_typ;
}

int set_br_conv (void)                  /* set breeding convergence           */
{
   if (chk_stat ())
   {
      chk_repeat ();
      Diff = d_par[0];
   }
   return err_typ;
}

int start_mean (void)                   /* start trait mean statement         */
{
   if (chk_stat ())
   {
      if (chk_trait ())
         start_trait_mean ();
   }
   return err_typ;
}

int start_effect (void)                 /* start trait.. effect.. statement   */
{
   if (chk_stat ())
   {
      if (chk_trait ())
      {
         if (chk_ff (1))
            if (chk_levels_ff ())
               start_trait_eff ();
      }
   }
   return err_typ;
}

int start_add_var (void)                /* start additive variance..          */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_start_var (0);
      if (chk_var ())
         set_start_var (0);

   }
   return err_typ;
}

int start_add_cor (void)                /* start additive correlation..       */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_start_cov (0);
      if (chk_covar ())
         set_start_cor (0);
   }
   return err_typ;
}

int start_add_cov (void)                /* start additive covariance..        */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_start_cov (0);
      if (chk_covar ())
         set_start_cov (0);
   }
   return err_typ;
}

int start_res_var (void)                /* start residual variance..          */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_start_var (1);
      if (chk_var ())
         set_start_var (1);
   }
   return err_typ;
}

int start_res_cor (void)                /* start residual correlation..       */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_start_cov (1);
      if (chk_covar ())
         set_start_cor (1);
   }
   return err_typ;
}

int start_res_cov (void)                /* start residual covariance..        */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_start_cov (1);
      if (chk_covar ())
         set_start_cov (1);
   }
   return err_typ;
}

int set_eigen (void)                    /* set eigenvalues ..                 */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_set_eigen ();
      set_set_eigen ();
   }
   return err_typ;
}

int set_allefrq_norm (void)             /* set normalized frequencies         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      alle_frq_norm = Y;
   }
   return err_typ;
}

int set_allefrq_mk (void)               /* set marker frequencies             */
{
   LOCUS_PTR p_locus;
   FREQ_PTR p_fnode;
   int   i;

   if (chk_stat ())
   {
      if (chk_locus (NCH, MARK_LOC))
      {
         p_fnode = alloc_frq ();
         stor_frq (p_fnode);
         for (i = 0; i < nk_par; i++)
         {
            if ((p_locus = chk_locus_node (j_par[i], NCH, MARK_LOC)))
               free_freq (p_locus);
            else
               p_locus = alloc_locus_node (j_par[i], NCH, MARK_LOC);
            stor_locus_data (j_par[i], p_locus, p_fnode, NCH, MARK_LOC);
         }
         chk_locus_norm ();
      }
   }
   return err_typ;
}

int set_allefrq_mk_c (void)             /* set chrom/marker frequencies       */
{
   LOCUS_PTR p_locus;
   FREQ_PTR p_fnode;
   int   i;

   if (chk_stat ())
   {
      if (chk_locus (CH, MARK_LOC))
      {
         p_fnode = alloc_frq ();
         stor_frq (p_fnode);
         for (i = 0; i < nk_par; i++)
         {
            if ((p_locus = chk_locus_node (j_par[i + 1], CH, MARK_LOC)))
               free_freq (p_locus);
            else
               p_locus = alloc_locus_node (j_par[i + 1], CH, MARK_LOC);
            stor_locus_data (j_par[i + 1], p_locus, p_fnode, CH, MARK_LOC);
         }
         chk_locus_norm ();
      }
   }
   return err_typ;
}

int set_allefrq_tl (void)               /* set tloc frequencies               */
{
   LOCUS_PTR p_locus;
   FREQ_PTR p_fnode;
   int   i;

   if (chk_stat ())
   {
      if (chk_locus (NCH, TLOC_LOC))
      {
         p_fnode = alloc_frq ();
         stor_frq (p_fnode);
         for (i = 0; i < nk_par; i++)
         {
            if ((p_locus = chk_locus_node (j_par[i], NCH, TLOC_LOC)))
               free_freq (p_locus);
            else
               p_locus = alloc_locus_node (j_par[i], NCH, TLOC_LOC);
            stor_locus_data (j_par[i], p_locus, p_fnode, NCH, TLOC_LOC);
         }
         chk_locus_norm ();
      }
   }
   return err_typ;
}

int set_allefrq_tl_imc (void)           /* set importance mc tloc frequencies */
{
   return err_typ;
}

int set_tr_tl_pene (void)               /* set traits ...tlocs... incom penet */
{                                       /* (there must be 3 penetrances).     */
   INCO_PENE_PTR p_pene;
   int   t_no, tl_no;
   int   i1;

   if (chk_stat ())
   {
      if (chk_inco_pene ())
         for (i1 = 0; i1 < nk_par / 2; i1++)
         {
            t_no = j_par[i1];
            tl_no = j_par[nk_par / 2 + i1];
            if (!(p_pene = chk_inco_pene_node (t_no, tl_no)))
               p_pene = alloc_inco_pene_node (t_no, tl_no);
            else
               free_inco_pene_array (p_pene);
            alloc_inco_pene_array (p_pene);
            stor_inco_pene_array (p_pene);
         }
   }

   return err_typ;
}

int set_tr_tl_geno (void)               /* set traits ...tlocs... genotyp mns */
{                                       /* (there must be 3 means).           */
   GENO_MEANS_PTR p_means;
   int   t_no, tl_no;
   int   i1;

   if (chk_stat ())
   {
      if (chk_geno_means ())
         for (i1 = 0; i1 < nk_par / 2; i1++)
         {
            t_no = j_par[i1];
            tl_no = j_par[nk_par / 2 + i1];
            if (!(p_means = chk_geno_means_node (t_no, tl_no)))
               p_means = alloc_geno_means_node (t_no, tl_no);
            else
               free_geno_means_array (p_means);
            alloc_geno_means_array (p_means);
            stor_geno_means_array (p_means);
         }
   }

   return err_typ;
}

int set_tr_tl_joint_geno (void)         /* set traits ...tlocs.. join geno mn */
{                                       /* (there must be 9 means).           */
   GENO_MEANS_PTR p_means, p_j;
   int   t_no, tl_no, joined_tlno;

   if (chk_stat ())
   {
      if (chk_joint_geno_means ())
      {
         t_no = j_par[0];
         tl_no = j_par[1];
         joined_tlno = j_par[2];

         if (!(p_means = chk_joint_geno_means_node (t_no, tl_no)))
            p_means = alloc_geno_means_node (t_no, tl_no);
         else
            free_geno_means_array (p_means);
         alloc_geno_means_array (p_means);
         stor_geno_means_array (p_means);

         p_means->joint = TRUE;
         p_means->joined_tlno = joined_tlno;

         if ((p_j = chk_joint_geno_means_node (t_no, joined_tlno)))
            free_geno_means_array (p_j);

/* The means in the joined to geno_means_node don't need to be initialized.   */
/* By leaving the number of means in this node at 0, the means should not be  */
/* referred to.  The tloc number set by alloc_geno_means_node will not be the */
/* joined to tloc number, so change it to the joined to tloc number.          */

         else
         {
            p_j = alloc_geno_means_node (t_no, joined_tlno);
            p_j->tloc_no = joined_tlno;
         }

         p_j->joint = TRUE;
         p_j->joined_tlno = tl_no;

/* Both the geno_means_node's produced by using the "set trait K1 for tlocs   */
/* L1 L2 joint genotype means X11 ... X33" statement need to be able to refer */
/* to each other.  This is done through the ->p_joined pointers in each node. */

         p_means->p_joined = p_j;
         p_j->p_joined = p_means;

         GxGInteraction = TRUE;
      }
   }

   return err_typ;
}

int set_addi_var (void)                 /* set additive variances             */
{
   VARI_PTR p_vari;
   int   i;

   if (chk_stat ())
   {
      if (chk_vari ())
         for (i = 0; i < nk_par; i++)
         {
            if (!(p_vari = chk_vari_node (j_par[i], AV)))
               p_vari = alloc_vari_node (j_par[i], AV);
            stor_vari_data (i, p_vari);
         }
   }
   return err_typ;
}

int set_resi_var (void)                 /* set residual variances             */
{
   VARI_PTR p_vari;
   int   i;

   if (chk_stat ())
   {
      if (chk_vari ())
         for (i = 0; i < nk_par; i++)
         {
            if (!(p_vari = chk_vari_node (j_par[i], RV)))
               p_vari = alloc_vari_node (j_par[i], RV);
            stor_vari_data (i, p_vari);
         }
   }
   return err_typ;
}

int set_mark_seeds (void)               /* set marker seeds                   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      chk_seeds ();
      MarkSeedSet = Y;
      MarkSeedC = u_par[0];
      MarkSeedT = u_par[1];
   }
   return err_typ;
}

int set_trait_seeds (void)              /* set trait seeds                    */
{
   if (chk_stat ())
   {
      chk_repeat ();
      chk_seeds ();
      TraitSeedSet = Y;
      TraitSeedC = u_par[0];
      TraitSeedT = u_par[1];
   }
   return err_typ;
}

int set_sampl_seeds (void)              /* set sampler seeds                  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      chk_seeds ();
      SamplSeedSet = Y;
      SamplSeedC = u_par[0];
      SamplSeedT = u_par[1];
   }
   return err_typ;
}

int set_perm_seeds (void)               /* set permutation seeds              */
{
   if (chk_stat ())
   {
      chk_repeat ();
      chk_seeds ();
      PermSeedSet = Y;
      PermSeedC = u_par[0];
      PermSeedT = u_par[1];
   }
   return err_typ;
}

int set_mark_dat (void)                 /* set marker data                    */
{
   MARKDAT_PTR p_mdnode;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_mrkdat (NCH))
      {                                 /* tests for existing node            */
         if (!(p_mdnode = chk_mrkdat_node (NCH)))
            p_mdnode = alloc_mrkdat_node (NCH);
         else
            free_mrkdat_list (p_mdnode);
         alloc_mrkdat_list (p_mdnode);
         stor_mrkdat (p_mdnode, NCH);   /* stores data in node and in list    */
      }
   }
   return err_typ;
}

int set_cmark_dat (void)                /* set marker data for chromosome     */
{
   MARKDAT_PTR p_mdnode;

   if (chk_stat ())
      if (chk_mrkdat (CH))
      {                                 /* tests for existing node            */
         if (!(p_mdnode = chk_mrkdat_node (CH)))
            p_mdnode = alloc_mrkdat_node (CH);
         else
            free_mrkdat_list (p_mdnode);
         alloc_mrkdat_list (p_mdnode);
         stor_mrkdat (p_mdnode, CH);    /*stores data in node and in list     */
      }
   return err_typ;
}

int set_cmark_nams (void)               /* set marker names for chromosome    */
{
   MARKNAM_PTR p_mnnode;

   if (chk_stat ())
      if (chk_mrknam (CH))
      {                                 /* tests for existing node            */
         if (!(p_mnnode = chk_mrknam_node (CH)))
            p_mnnode = alloc_mrknam_node (CH);
         else
            free_mrknam_list (p_mnnode);
         alloc_mrknam_list (p_mnnode);
         stor_mrknam (p_mnnode, CH);    /*stores names in node and in list    */
      }
   return err_typ;
}

int set_mark_nams (void)                /* set marker names for chromosome    */
{
   MARKNAM_PTR p_mnnode;

   if (chk_stat ())
      if (chk_mrknam (NCH))
      {                                 /* tests for existing node            */
         if (!(p_mnnode = chk_mrknam_node (NCH)))
            p_mnnode = alloc_mrknam_node (NCH);
         else
            free_mrknam_list (p_mnnode);
         alloc_mrknam_list (p_mnnode);
         stor_mrknam (p_mnnode, NCH);   /*stores names in node and in list    */
      }
   return err_typ;
}

int set_prbgam (void)                   /* set proband gametes                */
{
   PROBGAM_PTR p_pgnode;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_prbgam (NCMP, NSC))
      {
         if (!(p_pgnode = chk_prbgam_node (NCMP, NSC)))
            p_pgnode = alloc_prbgam_node (NCMP, NSC);
         else
            free_prbgam_list (p_pgnode);
         alloc_prbgam_list (p_pgnode);
         stor_prbgam (p_pgnode, NCMP, NSC);
      }
   }
   return err_typ;
}

int set_cp_prbgam (void)                /* set component proband gametes      */
{
   PROBGAM_PTR p_pgnode;

   if (chk_stat ())
      if (chk_prbgam (CMP, NSC))
      {
         if (!(p_pgnode = chk_prbgam_node (CMP, NSC)))
            p_pgnode = alloc_prbgam_node (CMP, NSC);
         else
            free_prbgam_list (p_pgnode);
         alloc_prbgam_list (p_pgnode);
         stor_prbgam (p_pgnode, CMP, NSC);
      }
   return err_typ;
}

int set_sc_prbgam (void)                /* set scoreset proband gametes       */
{
   PROBGAM_PTR p_pgnode;

   if (chk_stat ())
      if (chk_prbgam (NCMP, SC))
      {
         if (!(p_pgnode = chk_prbgam_node (NCMP, SC)))
            p_pgnode = alloc_prbgam_node (NCMP, SC);
         else
            free_prbgam_list (p_pgnode);
         alloc_prbgam_list (p_pgnode);
         stor_prbgam (p_pgnode, NCMP, SC);
      }
   return err_typ;
}

int set_cs_prbgam (void)                /* set component scoreset proband     */
{                                       /* gametes                            */
   PROBGAM_PTR p_pgnode;

   if (chk_stat ())
      if (chk_prbgam (CMP, SC))
      {
         if (!(p_pgnode = chk_prbgam_node (CMP, SC)))
            p_pgnode = alloc_prbgam_node (CMP, SC);
         else
            free_prbgam_list (p_pgnode);
         alloc_prbgam_list (p_pgnode);
         stor_prbgam (p_pgnode, CMP, SC);
      }
   return err_typ;
}

#if 0
int set_cp_affnam (void)                /* set component affected individuals */
{
   INDNAM_PTR p_affindnode;

   if (chk_stat ())
      if (chk_indnam (CMP))
      {
         if (!(p_affindnode = chk_indnam_node (CMP, AFFIND)))
            p_affindnode = alloc_indnam_node (CMP, AFFIND);
         else
            free_indnam_list (p_affindnode);
         alloc_indnam_list (p_affindnode);
         stor_indnam (p_affindnode, CMP);
      }
   return err_typ;
}

int set_affnam (void)                   /* set affected individuals           */
{
   INDNAM_PTR p_affindnode;

   if (chk_stat ())
      if (chk_indnam (NCMP))
      {
         if (!(p_affindnode = chk_indnam_node (NCMP, AFFIND)))
            p_affindnode = alloc_indnam_node (NCMP, AFFIND);
         else
            free_indnam_list (p_affindnode);
         alloc_indnam_list (p_affindnode);
         stor_indnam (p_affindnode, NCMP);
      }
   return err_typ;
}
#endif

int set_loc_wind (void)                 /* set locus window                   */
{
   LOCWND_PTR p_wnd;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_wind (NCH))
      {                                 /* checks for node present            */
         if (!(p_wnd = chk_wind_node (NCH)))
            p_wnd = alloc_wind_node (NCH);
         stor_wind (p_wnd, NCH);        /* saves window size                  */
      }
   }
   return err_typ;
}

int set_cloc_wind (void)                /* set chromosome locus window        */
{
   LOCWND_PTR p_wnd;

   if (chk_stat ())
      if (chk_wind (CH))
      {                                 /* checks for node present for chrom  */
         if (!(p_wnd = chk_wind_node (CH)))
            p_wnd = alloc_wind_node (CH);
         stor_wind (p_wnd, CH);         /* saves window size                  */
      }
   return err_typ;
}

int set_windpat (void)                  /* set window patterns                */
{
   WNDPAT_PTR p_wpnode;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_wndpat (NCMP, NSC))
      {
         if (!(p_wpnode = chk_wndpat_node (NCMP, NSC)))
            p_wpnode = alloc_wndpat_node (NCMP, NSC);
         else
            free_wndpat_set (p_wpnode);
         alloc_wndpat_set (p_wpnode);
         stor_wndpat (p_wpnode, NCMP, NSC);
      }
   }
   return err_typ;
}

int set_cp_windpat (void)               /* set component window patterns      */
{
   WNDPAT_PTR p_wpnode;

   if (chk_stat ())
      if (chk_wndpat (CMP, NSC))
      {
         if (!(p_wpnode = chk_wndpat_node (CMP, NSC)))
            p_wpnode = alloc_wndpat_node (CMP, NSC);
         else
            free_wndpat_set (p_wpnode);
         alloc_wndpat_set (p_wpnode);
         stor_wndpat (p_wpnode, CMP, NSC);
      }
   return err_typ;
}

int set_sc_windpat (void)               /* set scoreset window patterns       */
{
   WNDPAT_PTR p_wpnode;

   if (chk_stat ())
      if (chk_wndpat (NCMP, SC))
      {
         if (!(p_wpnode = chk_wndpat_node (NCMP, SC)))
            p_wpnode = alloc_wndpat_node (NCMP, SC);
         else
            free_wndpat_set (p_wpnode);
         alloc_wndpat_set (p_wpnode);
         stor_wndpat (p_wpnode, NCMP, SC);
      }
   return err_typ;
}

int set_cs_windpat (void)               /* set component scoreset             */
{                                       /* window patterns                    */
   WNDPAT_PTR p_wpnode;

   if (chk_stat ())
      if (chk_wndpat (CMP, SC))
      {
         if (!(p_wpnode = chk_wndpat_node (CMP, SC)))
            p_wpnode = alloc_wndpat_node (CMP, SC);
         else
            free_wndpat_set (p_wpnode);
         alloc_wndpat_set (p_wpnode);
         stor_wndpat (p_wpnode, CMP, SC);
      }
   return err_typ;
}

int set_reco_frq (void)                 /* set recombination frequencies      */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_reco_frq ())
      {
         if (!NInbreedRF)
            InbreedRF = alloc_reco_frq ();
         else
            free_reco_frq (&InbreedRF);

         stor_reco_frq (&NInbreedRF, InbreedRF);
      }
   }
   return err_typ;
}

int set_ascent_iter (void)              /* set ascent iterations              */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERA_NAI;
      NHybAscentIt = j_par[0];
   }
   return err_typ;
}

int set_curv_iter (void)                /* set curvature iterations           */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERA_NCI;
      NHybCurvIt = j_par[0];
   }
   return err_typ;
}

int set_grad_iter (void)                /* set gradient iterations            */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERA_NGI;
      NSAGradIt = j_par[0];
   }
   return err_typ;
}

int set_lrt_iter (void)                 /* set LRT statistics iterations      */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERA_NRI;
      NChiSqIt = j_par[0];
   }
   return err_typ;
}

int set_em_conv (void)                  /* set EM convergence                 */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (d_par[0] <= 0. || d_par[0] >= 1.)
         err_typ = ERA_PCE;
      EM_bound = d_par[0];
   }
   return err_typ;
}

int set_sa_conv (void)                  /* set SA convergence                 */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (d_par[0] <= 0. || d_par[0] >= 1.)
         err_typ = ERA_PCS;
      SA_bound = d_par[0];
   }
   return err_typ;
}

int set_geno_errat (void)               /* set genotyping error rate          */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (d_par[0] < 0. || d_par[0] >= 1.)
         err_typ = ERA_GER;
      GenoErrRat = d_par[0];
   }
   return err_typ;
}

int set_ibd_meas1 (void)                /* set ibd measures Spairs            */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 0;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas12 (void)               /* set ibd meas Spairs Srobdom        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 1;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas123 (void)              /* set ibd meas Spairs Srobdom Saff   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 1;
      SAffUnaffOn = 1;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_measall (void)              /* set ibd meas Sp Sr Sa Sl           */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 1;
      SAffUnaffOn = 1;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas124 (void)              /* set ibd meas Spairs Srobdom Slamb  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 1;
      SAffUnaffOn = 0;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas13 (void)               /* set ibd meas Spairs Saffect        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 0;
      SAffUnaffOn = 1;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas134 (void)              /* set ibd meas Spairs Saffect Slamb  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 0;
      SAffUnaffOn = 1;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas14 (void)               /* set ibd meas Spairs Slambda        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 0;
      SAffUnaffOn = 0;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas2 (void)                /* set ibd meas Srobdom               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 1;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas23 (void)               /* set ibd meas Srobdom Saffect       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 1;
      SAffUnaffOn = 1;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas234 (void)              /* set ibd meas Srobdom Saff Slambda  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 1;
      SAffUnaffOn = 1;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas24 (void)               /* set ibd meas Srobdom Slambda       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 1;
      SAffUnaffOn = 0;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas3 (void)                /* set ibd meas Saffect               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 0;
      SAffUnaffOn = 1;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas34 (void)               /* set ibd meas Saffect Slambda       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 0;
      SAffUnaffOn = 1;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas4 (void)                /* set ibd meas Slambda               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 0;
      SAffUnaffOn = 0;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas5 (void)                /* set ibd meas maxFGL                */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 0;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 1;
   }
   return err_typ;
}

int set_ibd_meas51 (void)               /* set ibd meas maxFGL Spairs         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 0;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 1;
   }
   return err_typ;
}

int set_ibd_meas512 (void)              /* set ibd meas maxFGL Spairs Srobdom */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 1;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 1;
   }
   return err_typ;
}

int set_ibd_meas52 (void)               /* set ibd meas  maxFGL Srobdom       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 1;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 1;
   }
   return err_typ;
}

int set_like_meas6 (void)               /* set like measures delta            */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (DeltaOn != UNDEF || Lambda_pOn != UNDEF)
         err_typ = ERA_RLS;
      DeltaOn = 1;
      Lambda_pOn = 0;
   }
   return err_typ;
}

int set_like_meas67 (void)              /* set like measiures delta lambda-p  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (DeltaOn != UNDEF || Lambda_pOn != UNDEF)
         err_typ = ERA_RLS;
      DeltaOn = 1;
      Lambda_pOn = 1;
   }
   return err_typ;
}

int set_like_meas7 (void)               /* set likelihood measures lambda-p   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (DeltaOn != UNDEF || Lambda_pOn != UNDEF)
         err_typ = ERA_RLS;
      DeltaOn = 0;
      Lambda_pOn = 1;
   }
   return err_typ;
}

int set_ibd_test1 (void)                /* set ibd tests normal               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (NormalEnable != UNDEF || PermutEnable != UNDEF)
         err_typ = ERA_RIT;
      NormalEnable = 1;
      PermutEnable = 0;
   }
   return err_typ;
}

int set_ibd_test12 (void)               /* set ibd tests normal               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (NormalEnable != UNDEF || PermutEnable != UNDEF)
         err_typ = ERA_RIT;
      NormalEnable = 1;
      PermutEnable = 1;
   }
   return err_typ;
}

int set_ibd_test2 (void)                /* set ibd tests permutation          */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (NormalEnable != UNDEF || PermutEnable != UNDEF)
         err_typ = ERA_RIT;
      NormalEnable = 0;
      PermutEnable = 1;
   }
   return err_typ;
}

int set_like_test3 (void)               /* set likehd-rat tests chi-squared   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (ChiSqEnable != UNDEF)
         err_typ = ERA_RLT;
      ChiSqEnable = 1;
   }
   return err_typ;
}

int set_ibd_perm (void)                 /* set ibd permutations               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERA_SPI;
      else if (j_par[0] < 50)
         err_typ = ERA_RPI;

      NPermutIt = j_par[0];
   }
   return err_typ;
}

int set_like_grdpts (void)              /* set likehd-rat lambda-p gridpoints */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1 || j_par[1] < 1)
         err_typ = ERA_BGP;

      NumOfSegL = j_par[0];
      NumOfSegP = j_par[1];
   }
   return err_typ;
}

int set_map_mistyp (void)               /* set map estim model with mistyping */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (MapEstMod != UNDEF)
         err_typ = ERA_MES;
      MapEstMod = MISTYP;
   }
   return err_typ;
}

int set_map_nmistyp (void)              /* set map estim model w/o mistyping  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (MapEstMod != UNDEF)
         err_typ = ERA_MES;
      MapEstMod = !MISTYP;
   }
   return err_typ;
}

int set_trait_logistic (void)           /* set trait discrete with covariate
                                         * for HB, HD data                    */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PedDatTyp != UNDEF)
         err_typ = ERR_STT;
      PedDatTyp = DISCR_COVAR;
      TraitTyp = DISCRETE;
   }
   return err_typ;
}

int set_mcmc_markers_only (void)        /* Set mcmc markers only.             */
{
   if (chk_stat ())
   {
      chk_repeat ();
      McmcMarkersOnly = TRUE;
   }
   return err_typ;
}

int set_global_mcmc (void)              /* Set global mcmc.                   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      GlobalMcmc = TRUE;
   }
   return err_typ;
}

int set_debugmain (void)                /* Set debug flag for main program.   */
{
   if (chk_stat ())
      chk_repeat ();
   DebugMain = Y;

   return err_typ;
}

int set_debugcmf (void)                 /* Set debug flag for CMF library.    */
{
   if (chk_stat ())
      chk_repeat ();
   DebugCMF = Y;

   return err_typ;
}

int set_debuggldisp (void)              /* Set debug flag for GLDisp library. */
{
   if (chk_stat ())
      chk_repeat ();
   DebugGLDisp = Y;

   return err_typ;
}

int set_debugmarkrs (void)              /* Set debug flag for Markers library. */
{
   if (chk_stat ())
      chk_repeat ();
   DebugMarkers = Y;

   return err_typ;
}

int set_debugnghds (void)               /* Set debug flag for Nghds library.  */
{
   if (chk_stat ())
      chk_repeat ();
   DebugNghds = Y;

   return err_typ;
}

int set_debugpars (void)                /* Set debug flag for Pars library.   */
{
   if (chk_stat ())
      chk_repeat ();
   DebugPars = Y;

   return err_typ;
}

int set_debugpedchk (void)              /* Set debug flag for Pedchk library. */
{
   if (chk_stat ())
      chk_repeat ();
   DebugPedchk = Y;

   return err_typ;
}

int set_debugpeel (void)                /* Set debug flag for Peel library.   */
{
   if (chk_stat ())
      chk_repeat ();
   DebugPeel = Y;

   return err_typ;
}

int set_debugquant (void)               /* Set debug flag for Quant library.  */
{
   if (chk_stat ())
      chk_repeat ();
   DebugQuant = Y;

   return err_typ;
}

int set_debugrans (void)                /* Set debug flag for Rans library.   */
{
   if (chk_stat ())
      chk_repeat ();
   DebugQuant = Y;

   return err_typ;
}

int set_debugsample (void)              /* Set debug flag for Sample library. */
{
   if (chk_stat ())
      chk_repeat ();
   DebugSample = Y;

   return err_typ;
}

int set_debugstuff (void)               /* Set debug flag for Stuff library.  */
{
   if (chk_stat ())
      chk_repeat ();
   DebugStuff = Y;

   return err_typ;
}

int set_debugtwoqtl (void)              /* Set debug flag for Twoqtl library. */
{
   if (chk_stat ())
      chk_repeat ();
   DebugTwoqtl = Y;

   return err_typ;
}

int set_printlevel (void)               /* Set print level.                   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 0)
         err_typ = ERR_PRI;
      PrintLevel = j_par[0];
   }
   return err_typ;
}

int set_debuglevel (void)               /* Set debug level.                   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 0)
         err_typ = ERR_DEB;
      DebugLevel = j_par[0];
   }
   return err_typ;
}
