/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*
 * proc_seeds() calls functions to set parameters for seeds
 */

int proc_seeds (void)
{
   if (!test_seeds ())
      if (proc_err ())
         return FALSE;

   if (!test_oseeds ())
      if (proc_err ())
         return FALSE;

   if (n_err)
      return proc_end ();

   return TRUE;
}

/* --------- test fncs:  test results of processing seed statements --------- */

int test_seeds (void)
/* Determines whether seeds were given.                                       */
{
   if (GENEDROP)
   {
      if (!MarkSeedSet)                 /* Tests for marker seeds set.        */
      {
         MarkSeedC = CONGRUENTIAL_SEED;
         MarkSeedT = TAUSWORTHE_SEED;
         err_typ = ERD_SSD;
      }

      if (!TraitSeedSet)                /* Tests for trait seeds set.         */
      {
         TraitSeedC = 0;
         TraitSeedT = 0;
      }
   }

   if (MARKERDROP)
   {
      if (!TraitSeedSet)                /* Tests for trait seeds set.         */
      {
         TraitSeedC = CONGRUENTIAL_SEED;
         TraitSeedT = TAUSWORTHE_SEED;
         err_typ = ERD_STD;
      }

      if (!MarkSeedSet)                 /* Tests for marker seeds set.        */
      {
         MarkSeedC = 0;
         MarkSeedT = 0;
      }
   }

   if (LM_IBDTESTS)
   {
      if (PermutEnable)                 /* Tests for permutation tests.       */
      {
         if (!PermSeedSet)              /* Tests for permutation seeds set.   */
         {
            PermSeedC = CONGRUENTIAL_SEED;
            PermSeedT = TAUSWORTHE_SEED;
            err_typ = ERA_SSP;
         }

         if (!SamplSeedSet)             /* Tests for sample seeds set.        */
         {
            SamplSeedC = 0;
            SamplSeedT = 0;
         }
      }
      else                              /* Sets sampling seeds only.          */
      {
         PermSeedC = 0;
         PermSeedT = 0;

         if (!SamplSeedSet)             /* Tests for sample seeds set.        */
         {
            SamplSeedC = CONGRUENTIAL_SEED;
            SamplSeedT = TAUSWORTHE_SEED;
            err_typ = ERA_SSS;
         }
      }
   }

   if (LM_TWOQTL)
   {
      if (!TraitSeedSet)                /* Tests for trait seeds set.         */
      {
         TraitSeedC = CONGRUENTIAL_SEED;
         TraitSeedT = TAUSWORTHE_SEED;
         err_typ = ERD_STD;
      }

      if (!SamplSeedSet)                /* Tests for sampler seeds set.       */
      {
         SamplSeedC = CONGRUENTIAL_SEED;
         SamplSeedT = TAUSWORTHE_SEED;
         err_typ = ERA_SSS;
      }
   }

   if (IBDDROP || LM_MAP || LM_PVAL || LM_AUTO
       || LM_LODS || LM_BAYES || LM_MARKERS || LM_SCHNELL)
   {
      if (!SamplSeedSet)                /* Tests for sampler seeds set.       */
      {
         SamplSeedC = CONGRUENTIAL_SEED;
         SamplSeedT = TAUSWORTHE_SEED;
         err_typ = ERA_SSS;
      }
   }

   if (!err_typ)
      return TRUE;
   else
      return FALSE;
}

int test_oseeds (void)
{
   pft = get_filent (OSEED);

   if (GENEDROP || MARKERDROP)
   {
      if (pft->fname && !MarkSeedsOut && !TraitSeedsOut)
      {
         if (NLoci - NTraits)
            MarkSeedsOut = Y;
         if (NTraits)
            TraitSeedsOut = Y;
      }
      if (!pft->fname && MarkSeedsOut)
      {
         MarkSeedsOut = N;
         err_typ = ERD_OSM;
      }
      if (!pft->fname && TraitSeedsOut)
      {
         TraitSeedsOut = N;
         err_typ = ERD_OST;
      }
   }

   else if (IBDDROP || LM_MAP || LM_PVAL || LM_AUTO
            || LM_LODS || LM_BAYES || LM_MARKERS || LM_SCHNELL || LM_TWOQTL)
   {
      if (pft->fname)
         SamplSeedsOut = Y;
      else
         SamplSeedsOut = N;
   }

   else if (LM_IBDTESTS)
   {
      if (pft->fname && !SamplSeedsOut && !PermSeedsOut)
      {
         SamplSeedsOut = Y;
         PermSeedsOut = Y;
      }
      if (!pft->fname && SamplSeedsOut)
      {
         SamplSeedsOut = N;
         err_typ = ERA_USS;
      }
      if (!pft->fname && PermSeedsOut)
      {
         PermSeedsOut = N;
         err_typ = ERA_USP;
      }
   }

   if (!err_typ)
      return TRUE;
   else
      return FALSE;
}
