/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "pedchk.h"

/* opens and reads the pedigree file.  Stores the data in the pedchk    */
/* data structures.                                                     */
/* on return, leaves the keytable sorted in chronological order         */

int chk_pedfil (void)
{
   FILE *pedF;

   if (!(pedF = open_ped ()))           /* opens pedigree file          */
      return !OK;

   allocPed ();                         /* allocates arrays             */

   if (!readPed (pedF))                 /* reads pedigree file          */
      return !OK;

   if (!checkPed (pedF))                /* reports pedigree size        */
      return !OK;
   fclose (pedF);

   allocKey ();                         /* allocates key table          */

   storKey ();                          /* stores names in key table    */

   sortNam ();                          /* sorts key table on name      */

   if (!checkNam ())                    /* checks for dupicate names    */
      return !OK;

   findParents ();                      /* stores ix's to parents in
                                         * key table */

   if (!checkParents ())                /* checks for missing parents   */
      return !OK;

   assignGender ();                     /* if requested, assigns gender */

   sortIx ();                           /* sorts key table on rec. no.  */

   if (!checkGender ())                 /* checks parents for correct   */
      return !OK;                       /* gender                       */

   assignChron ();                      /* finds chronological order
                                         * and stores no. in key table  */

   if (!checkChron ())                  /* checks for chronological     */
      return !OK;                       /* order found                  */

   assignComp ();                       /* finds the components         */

   sortComp ();                         /* sorts key table by component
                                         * and chronological order      */

   allocComp ();                        /* allocates component table    */

   storComp ();                         /* stores component table info  */

   prntComp ();                         /* prints component info        */

   reassignChron ();                    /* sets chonological order by   * 
                                         * component in key table       */

   sortChron ();                        /* sorts by component, then     * 
                                         * chronological order          */
   return OK;
}
