/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "rans.h"

/* functions for managing seeds for MORGAN V2             
 * see source file rans.c for the random number generator used
 * 
 * functions are: 
 *   setSeeds()  
 *   getSeeds()
 *   assgnSeeds()
 *   writSeeds()
 */

void setSeeds (int seedTyp)
{
   uint32 *pSeedC = NULL;
   uint32 *pSeedT = NULL;

   switch (seedTyp)
   {
       case MARK:
          pSeedC = &MarkSeedC;
          pSeedT = &MarkSeedT;
          fprintf (stdout, " Hexadecimal marker seeds:");
          break;

       case TRAIT:
          pSeedC = &TraitSeedC;
          pSeedT = &TraitSeedT;
          fprintf (stdout, " Hexadecimal trait seeds:");
          break;

       case SAMPL:
          pSeedC = &SamplSeedC;
          pSeedT = &SamplSeedT;
          fprintf (stdout, " Hexadecimal sampler seeds:");
          break;

       case PERM:
          pSeedC = &PermSeedC;
          pSeedT = &PermSeedT;
          fprintf (stdout, " Hexadecimal permutation seeds:");
          break;

       default:
          ;
   }

   setsd (*pSeedC, *pSeedT);

   fprintf (stdout, "  %#x %#x\n\n", *pSeedC, *pSeedT);

   return;
}

void getSeeds (int seedTyp)
{
   uint32 *pSeedC = NULL;
   uint32 *pSeedT = NULL;

   switch (seedTyp)
   {
       case MARK:
          pSeedC = &MarkSeedC;
          pSeedT = &MarkSeedT;
          break;

       case TRAIT:
          pSeedC = &TraitSeedC;
          pSeedT = &TraitSeedT;
          break;

       case SAMPL:
          pSeedC = &SamplSeedC;
          pSeedT = &SamplSeedT;
          break;

       case PERM:
          pSeedC = &PermSeedC;
          pSeedT = &PermSeedT;
          break;

       default:
          ;
   }
   getsd (pSeedC, pSeedT);

   return;
}

void assgnSeeds (int seedTyp)
{
   switch (seedTyp)
   {
       case MARK:
          if (!MarkSeedSet)
          {
             MarkSeedC = TraitSeedC;
             MarkSeedT = TraitSeedT;
          }
          break;

       case TRAIT:
          if (!TraitSeedSet)
          {
             TraitSeedC = MarkSeedC;
             TraitSeedT = MarkSeedT;
          }
          break;

       case SAMPL:
          if (!SamplSeedSet)
          {
             SamplSeedC = PermSeedC;
             SamplSeedT = PermSeedT;
          }
          break;

       default:
          ;
   }

   return;
}

void writSeeds (FILE * oSeedF)
{
   if (MarkSeedsOut)
      fprintf (oSeedF, " set marker seeds %#x %#x\n", MarkSeedC, MarkSeedT);
   if (TraitSeedsOut)
      fprintf (oSeedF, " set trait seeds  %#x %#x\n", TraitSeedC, TraitSeedT);
   if (SamplSeedsOut)
      fprintf (oSeedF, " set sampler seeds  %#x %#x\n", SamplSeedC, SamplSeedT);
   if (PermSeedsOut)
      fprintf (oSeedF, " set permutation seeds  %#x %#x\n",
               PermSeedC, PermSeedT);
   return;
}
