/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "lsample.h"
#include "mulmsample.h"

/* cmp = 0: Consider all components simultaneously.                           */
/* cmp > 0: Indicates which component to consider.                            */
/* mul = 0: M-sampler; otherwise, MM-sampler.                                 */

void do_mcmc_sampling (int cmp, int mul, int ntp, int *p_scored_iterations,
                       double **prob, double *mcmc_lod, double *mcmc_sd,
                       CTP_PTR p_ctp, RLD_PTR p_rld, MSD_PTR p_msd,
                       FILE * oScorF)
{
   int   bc;                            /* Current batch indicator.           */

   int   iteration;
   int   c1;
   int   p1;

   prntStatusBMC ();                    /* Print status for beginning MCMC.   */

   bc = 0;                              /* Set current batch indicator to 0.  */

   /* Setup variables for LM-multiple sampling: LPT Apr 2006.                 */
   setup_sampling_cmp (cmp, mul, NMarks);

   /* Now do burn-in iterations.                                              */

   printf (" Now starting %d iterations of burn-in\n", NBrnIt);
   fflush (stdout);

   for (iteration = 1; iteration <= NBrnIt; ++iteration)
   {
      if (!(iteration % ChkIt))
      {
         printf (" Starting burn-in iteration %d\n", iteration);
         fflush (stdout);
      }
      sampling_cmp (iteration, cmp, 1, mul, NMarks, nFGL, prob);
   }

   /* Now do the regular MCMC sampling, and score the results.                */

   printf (" Now starting %d iterations of MCMC\n\n", NIt);
   fflush (stdout);

   for (iteration = 1; iteration <= NIt; ++iteration)
   {
      if (DebugMain)
         dbgXXGG ();

      /* MCMC sampling step or scan.                                          */

      sampling_cmp (iteration, cmp, 1, mul, NMarks, nFGL, prob);

      if (!(iteration % ScorIt))
      {

         /* Score total log-probabilities and recombinations.                 */
         if (cmp > 0)
         {
            full_seg_prob_cmp (NMarks, cmp - 1, p_msd->recounts[0],
                               p_msd->temp_log_prob_seg[0]);
            for (p1 = 1; p1 < NMarks; ++p1)
            {
               p_msd->tot_log_prob_pen[0][p1] += log (prob[cmp - 1][p1]);
               p_msd->tot_log_prob_seg[0][p1] +=
                  p_msd->temp_log_prob_seg[0][p1];
            }
            p_msd->tot_log_prob_pen[0][NMarks] += log (prob[cmp - 1][NMarks]);
         }
         else
         {
            for (c1 = 0; c1 < n_compnts; ++c1)
            {
               full_seg_prob_cmp (NMarks, c1, p_msd->recounts[c1],
                                  p_msd->temp_log_prob_seg[c1]);
               for (p1 = 1; p1 < NMarks; ++p1)
               {
                  p_msd->tot_log_prob_pen[c1][p1] += log (prob[c1][p1]);
                  p_msd->tot_log_prob_seg[c1][p1] +=
                     p_msd->temp_log_prob_seg[c1][p1];
               }
               p_msd->tot_log_prob_pen[c1][NMarks] += log (prob[c1][NMarks]);
            }
         }

         /* Score rb_likelihoods.                                             */

         if (bc < NBtch)
            do_rblik_cmp (cmp, ntp, bc, p_ctp, p_rld);
         ++(*p_scored_iterations);

         /* Every FilIt scored iterations, the current test positions log     */
         /* likelihoods need to be output to the interim scores file.         */

         /* Tests to see whether or not to output to scores file.             */
         if (FilIt && !(*p_scored_iterations % FilIt))
            /*Modified 5/2010 to output log-likelihoods of scored iterations. */
         {
            fprintf (oScorF, "\n log-likelihoods: iter %d, cmpt %d\n",
                     iteration, cmp);
            for (p1 = 1; p1 < ntp; ++p1)
               fprintf (oScorF, " %f", p_rld->cmpt_lik[p1]);
            fprintf (oScorF, "\n");
         }

         /* Start new batch if current batch is complete.                     */

         if (!(*p_scored_iterations % BatchSize))
            ++bc;
      }

   }

   printf ("\n");
   prntStatusFMC ();                    /* Print status for finished with MCMC. */

   return;
}

void set_batch_size (void)
{
   int   scored_iterations;             /* Number of scored iterations.       */

   scored_iterations = compute_scored_iterations ();
   BatchSize = scored_iterations / NBtch;

/* Check that the number of batches divides the number of scored iterations.  */
   if (BatchSize * NBtch != scored_iterations)
      give_warn (ERL_bt1, NBtch);

/* Make sure that BatchSize is a positive number.                             */
   if (BatchSize <= 0)
      give_up (ERL_bt2, NBtch);

   printf (" %d realizations will be scored, at scoring interval %d\n",
           scored_iterations, ScorIt);
   printf (" Scored realizations will be batched in %d sets of size %d\n",
           NBtch, BatchSize);

   return;
}

int compute_scored_iterations (void)
{
   int   scored_iterations;             /* Total number of iterations to be   */
   /*                                      used for lod score estimation.     */
   /*                                      For example, if 1000 iterations    */
   /*                                      are scored every 20, then          */
   /*                                      scored_iterations = 50.            */

   scored_iterations = NIt / ScorIt;
   if (scored_iterations * ScorIt != NIt)
      give_warn (ERR_err,
                 "Scoring interval does not divide realizations count");

   return scored_iterations;
}

CTP_PTR alloc_ctp (int npos)
{
/* double rec_left[2][NLocPos + 1], double rec_right[2][NLocPos + 1],         */
/* int    loc_left[NLocPos + 1],    int    loc_right[NLocPos + 1]             */

   CTP_PTR p_ctp;
   int   x1;

   p_ctp = malloc (sizeof (CTP));
   alloc_chk (WHERE, p_ctp);

   p_ctp->rec_left = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, p_ctp->rec_left);
   p_ctp->rec_rght = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, p_ctp->rec_rght);

   for (x1 = 0; x1 < 2; ++x1)
   {
      p_ctp->rec_left[x1] = malloc (npos * sizeof (double));
      alloc_chk (WHERE, p_ctp->rec_left[x1]);
      p_ctp->rec_rght[x1] = malloc (npos * sizeof (double));
      alloc_chk (WHERE, p_ctp->rec_rght[x1]);
   }

   p_ctp->loc_left = malloc (npos * sizeof (int));
   alloc_chk (WHERE, p_ctp->loc_left);
   p_ctp->loc_rght = malloc (npos * sizeof (int));
   alloc_chk (WHERE, p_ctp->loc_rght);

   return p_ctp;
}

/* Reallocate the four arrays in the cur_test_pos (current test position)     */
/* structure passed to this subroutine.  They  need to be reduced in size     */
/* from (NLocPos + 1), as they were originally allocated, to just ntp, as was */
/* determined by the set_eval_points subroutine.  The resulting arrays will   */
/* then be:                                                                   */
/*                                                                            */
/*    double rec_left[2][ntp], double rec_right[2][ntp],                      */
/*    int    loc_left[ntp],    int    loc_right[ntp]                          */

void realloc_ctp (int ntp, CTP_PTR p_ctp)
{
   int   x1;

   for (x1 = 0; x1 < 2; ++x1)
   {
      p_ctp->rec_left[x1] =
         realloc (p_ctp->rec_left[x1], ntp * sizeof (double));
      alloc_chk (WHERE, p_ctp->rec_left[x1]);
      p_ctp->rec_rght[x1] =
         realloc (p_ctp->rec_rght[x1], ntp * sizeof (double));
      alloc_chk (WHERE, p_ctp->rec_rght[x1]);
   }

   p_ctp->loc_left = realloc (p_ctp->loc_left, ntp * sizeof (int));
   alloc_chk (WHERE, p_ctp->loc_left);
   p_ctp->loc_rght = realloc (p_ctp->loc_rght, ntp * sizeof (int));
   alloc_chk (WHERE, p_ctp->loc_rght);

   return;
}

void free_ctp (CTP_PTR p_ctp)
{
   free_2d ((void *) &p_ctp->rec_left);
   free_2d ((void *) &p_ctp->rec_rght);
   free_1d ((void *) &p_ctp->loc_left);
   free_1d ((void *) &p_ctp->loc_rght);
   free (p_ctp);

   return;
}

/******************************************************************************/
/*                                                                            */
/* The alloc_rld subroutine is called to allocate and initialize to 0.0's     */
/* whichever of the 4 double precision floating point arrays selected by the  */
/* caller from among the arrays used to hold the cumulative rb_likelihoods    */
/* within batches and by test position, along with their corresponding monte  */
/* carlo standard deviations.                                                 */
/*                                                                            */
/* The first argument to this subroutine is a null-terminated string.  This   */
/* string is used to indicate which of the arrays is to be allocated and      */
/* initialized by this subroutine.  The following table includes the type of  */
/* data in each array, the name of the array, the dimensions of the array and */
/* the options code letter included as a character in the first argument to   */
/* tell the subroutine to allocate and initialize the array.                  */
/*                                                                            */
/* array data type   array name       dimensions       options code letter    */
/*                                                                            */
/*     double        cmpt_lik    [NTPos + NMarks + 1]          'C'            */
/*     double        rb_prob     [NTPos + NMarks + 1]          'P'            */
/*     double        rb_prbm     [NBtch][NTPos + NMarks + 1]   'B'            */
/*     double        rb_sd       [NTPos + NMarks + 1]          'S'            */
/*                                                                            */
/******************************************************************************/

RLD_PTR alloc_rld (const char *options, int npos, int nb)
{
   RLD_PTR p_rld;

   p_rld = malloc (sizeof (RLD));
   alloc_chk (WHERE, p_rld);
   p_rld->base_lik = 0.0;

   if (strchr (options, 'C'))
      p_rld->cmpt_lik = alloc_dbl1d (npos);
   else
      p_rld->cmpt_lik = NULL;

   if (strchr (options, 'P'))
      p_rld->rb_prob = alloc_dbl1d (npos);
   else
      p_rld->rb_prob = NULL;

   if (strchr (options, 'B'))
      p_rld->rb_prbm = alloc_dbl2d (nb, npos);
   else
      p_rld->rb_prbm = NULL;

   if (strchr (options, 'S'))
      p_rld->rb_sd = alloc_dbl1d (npos);
   else
      p_rld->rb_sd = NULL;

   return p_rld;
}

void free_rld (RLD_PTR p_rld)
{
   free_1d ((void *) &p_rld->cmpt_lik);
   free_1d ((void *) &p_rld->rb_prob);
   free_2d ((void *) &p_rld->rb_prbm);
   free_1d ((void *) &p_rld->rb_sd);
   free (p_rld);

   return;
}

MSD_PTR alloc_msd (void)
{
   MSD_PTR p_msd;

   p_msd = malloc (sizeof (MSD));
   alloc_chk (WHERE, p_msd);

   p_msd->recounts = alloc_int3d (n_compnts, NLoci, 6);
   p_msd->tot_log_prob_seg = alloc_dbl2d (n_compnts, NLoci);
   p_msd->tot_log_prob_pen = alloc_dbl2d (n_compnts, NLoci);
   p_msd->temp_log_prob_seg = alloc_dbl2d (n_compnts, NLoci);

   return p_msd;
}

void free_msd (MSD_PTR p_msd)
{
   free_3d ((void *) &p_msd->recounts);
   free_2d ((void *) &p_msd->tot_log_prob_seg);
   free_2d ((void *) &p_msd->tot_log_prob_pen);
   free_2d ((void *) &p_msd->temp_log_prob_seg);
   free (p_msd);

   return;
}
