/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "lsample.h"
#include "mulmsample.h"

/* cmp = 0: Consider all components simultaneously.                           */
/* cmp > 0: Indicates which component to consider.                            */
/* mul = 0: M-sampler; otherwise, MM-sampler.                                 */

void mcmc_lods (int cmp, int mul, int ntp, CTP_PTR p_ctp, double **prob,
                double *mcmc_lod, double *mcmc_sd)
{
   static FILE *oScorF = NULL;

   RLD_PTR p_rld;
   MSD_PTR p_msd;

   int   nctemp;                        /* Backup copy of original n_compnts. */
   int   scored_iterations;             /* Number of scored iterations.       */
   int   bc;                            /* Current batch indicator.           */

   int   c1;
   int   p1;

   /* Open output scores file, if requested to do so.                         */
   if (FilIt && !oScorF)
      if (!(oScorF = open_fil (OSCORE)))
         give_up (ERR_qit);

   nctemp = n_compnts;
   if (cmp)
      n_compnts = 1;

   /* Call set_batch_size to make sure that the scoring interval evenly       */
   /* divides the realizations count and that the BatchSize global variable   */
   /* is set to a positive number that evenly divides the number of scored    */
   /* iterations.                                                             */

   set_batch_size ();
   printf ("\n");
   fflush (stdout);

   /* Moved down initial peeling after alloc_rld etc: EAT July 2004.          */

   p_rld = alloc_rld ("CPBS", ntp, NBtch);

   /* Added EAT -- P(Y_T).                                                    */
   /* set_transmission (loci, 0, NMarks);                                     */

   set_simple_trans (loci, -1, -1, 0.5, 0.5, 0.5, 0.5);

   if (TraitTyp != QUANTITATIVE)
      p_rld->base_lik = locus_mjrpeel_cmp (cmp, 1.0, 1.0, 0, loci);
   else
      p_rld->base_lik = locus_mjrpeel_cmp (cmp, loci[0].ve, 1.0, 0, loci);

   /* Checks for zero probability in peeling operation.                       */
   if (p_rld->base_lik == -DBL_MAX)
      give_up_where (WHERE, ERS_zpp);

   printf (" Base log-likelihood log_10 P(Y_T) = %e\n",
           p_rld->base_lik / log (10.0));
   printf ("\n\n");
   fflush (stdout);

   p_msd = alloc_msd ();
   alloc_logprob ();

   /* Now call do_mcmc_sampling to do the MCMC sampling and data collection.  */

   scored_iterations = 0;               /*Initialize scored iterations count. */

   do_mcmc_sampling (cmp, mul, ntp, &scored_iterations, prob, mcmc_lod, mcmc_sd,
                     p_ctp, p_rld, p_msd, oScorF);

   /* Compute likelihood estimates.                                           */

   for (p1 = 1; p1 < ntp; ++p1)
   {
      p_rld->rb_prob[p1] = p_rld->rb_prob[p1] / (NBtch * BatchSize);
      for (bc = 0; bc < NBtch; ++bc)
         p_rld->rb_prbm[bc][p1] = p_rld->rb_prbm[bc][p1] / BatchSize;
   }

   /* Batch-means standard devation estimates.                                */
   /* Careful with component indexing in arrays!                              */

   for (p1 = 1; p1 < ntp; ++p1)
   {
      for (bc = 0; bc < NBtch; ++bc)
         p_rld->rb_sd[p1] += (p_rld->rb_prbm[bc][p1] - p_rld->rb_prob[p1]) *
            (p_rld->rb_prbm[bc][p1] - p_rld->rb_prob[p1]);
      p_rld->rb_sd[p1] = sqrt (p_rld->rb_sd[p1] / ((NBtch - 1) * NBtch));
   }

   /* Output results for this simulation/estimation set.                      */
   /* Moved down so likelihoods can also be by component: EAT July 2004.      */

   if (cmp > 0)
   {
      pr_cmpnthead (cmp - 1);
      pr_log_prob_seg (1, NMarks, -cmplist[cmp].nmeioses,
                       p_msd->tot_log_prob_seg[0], scored_iterations);
      pr_log_prob_pen (1, NMarks, p_msd->tot_log_prob_pen[0],
                       scored_iterations);
      pr_reco_counts (1, NMarks, p_msd->recounts[0]);
   }
   else
   {
      for (c1 = 0; c1 < n_compnts; ++c1)
      {
         pr_cmpnthead (c1);
         pr_log_prob_seg (1, NMarks, -cmplist[c1 + 1].nmeioses,
                          p_msd->tot_log_prob_seg[c1], scored_iterations);
         pr_log_prob_pen (1, NMarks, p_msd->tot_log_prob_pen[c1],
                          scored_iterations);
         pr_reco_counts (1, NMarks, p_msd->recounts[c1]);
      }
   }

   /* Now compute estimated lodscores at each test position along with their  */
   /* Monte Carlo standard deviations.                                        */

   for (p1 = 1; p1 < ntp; ++p1)
      if ((p_ctp->loc_left[p1] != p_ctp->loc_rght[p1]) || !NAforMarkers)
      {
         mcmc_lod[p1] = log10 (p_rld->rb_prob[p1]);
         mcmc_sd[p1] = p_rld->rb_sd[p1] / (p_rld->rb_prob[p1] * log (10.0));
      }

   free_logprob ();
   free_msd (p_msd);
   free_rld (p_rld);
   free_sampling_cmp ();

   n_compnts = nctemp;

   if (FilIt && (!cmp || (cmp == n_compnts)))
      fclose (oScorF);

   return;
}
