/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

   /*******************************************************************
    * L-Sampler Setup ; code by SCH, Summer 1999                      *
    *                                                                 *
    *   installed as separate subroutine; EAT, March 2000             *
    *******************************************************************/

#include "lsample.h"

void set_positions (int nloc)
{
   int   i, j, k;
   double zm, zf, rr;
   /* Set up positions for lsampler routines */

   zf = 0.0;
   zm = 0.0;
   j = 1;
   for (i = 0; i <= nloc; i++)
   {
      k = chromo[0][i];
      loci[k].chromosome = j;
      loci[k].malepos = zm;
      loci[k].fempos = zf;
      rr = xecomo[1][0][i];             /* male  map; EAT, 2/00 */
      if (rr == 0.5)
      {
         j++;
         zm = 0.0;
      }
      else
         zm += 100.0 * gendist (rr);    /* Get distance in cM */

      rr = xecomo[0][0][i];             /* female  map; EAT, 2/00 */
      if (rr == 0.5)
         zf = 0.0;                      /* bug fixed EAT March 2002 */
      else
         zf += 100.0 * gendist (rr);    /* Get distance in cM */
   }
   return;
}

void set_locpeel (int nl)
   /* move out initial setup from set_lsmpl, and from seq_imp,
      so it can be used independently by other progs.  EAT 8/2005    */
{
   static int done = 0;

   if (!done)
   {
      set_positions (nl);
      if (TraitTyp == GENOTYPIC)
         /* replaces prev program-dependent   mix-up: EAT 8/05  */
      {
         n_markers = nl + 1;
         n_tloci = 0;
      }
      else
      {
         n_markers = nl;
         n_tloci = 1;
      }

      /* Initialize peeling routines, find peeling sequence etc.
       * The first parameter is zero unless you're giving it
       * phase known data when it's a pointer to an int array of
       * size pedsize.  In this case individual i has phase known
       * data if the corresponding array entry is non-zero.
       * The second parameter is a quiet flag - if non-zero then no
       * messages are output *unless* an error is found.
       * (The third parameter is the genotype data array-- 
       *              NOW REMOVED-- EAT 09/2004.)
       */
      locus_peel_init (0, 1);
      peel_trace = 0;                   /* Set reporting level for peeling 
                                           functions */
      done = 1;
   }
   return;
}

void set_lsmpl (int rs, int mm, int nloc)
   /* mm flag is added to avoid sampling locus 0 if mm=1: EAT 7/2005 */
{
   double z;
   int   j;

   /* Set up positions for lsampler routines */
   set_locpeel (nloc);

   /* realise the initial switches if not already read above */
   if (rs == 0)
   {
      set_transmission (0, 0, 0);
      /* Initial sample is for unlinked loci. This is required. */
      fflush (stdout);
      for (j = mm; j <= nloc; j++)
      {
         /* Peel/sample routine.
          * First parameter was double array with residuals for
          *    quantitative trait (removed EAT 7/04)
          * First parameter is now residual variance 
          *       (used for QUANTITATIVE trait EAT 07/04).
          * Second parameter is lam - only used in mixt I think).
          * Third parameter is sample flag, 0 = peel only, >0 = sample
          * Fourth and fifth are GONE (added EAT 07/04; deleted 2010/06)
          * Last parameter is pointer to locus structure to be peeled/sampled
          *   EAT 09/04: the TraitTyp-dependent call is removed!
          */

         z = locus_mjrpeel (loci[j].ve, 1.0, 1, loci + j);

         /* checks for Zero probability in peeling operation                  */
         if (z == -DBL_MAX)
            give_up_where (WHERE, ERS_zpp);

         if (DebugSample)
            printf ("log-likelihood (Locus %d) = %-20f\n", j, z);
      }
   }

}
