/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include "lsample.h"

/* This subroutine sets the quantitative trait locus information for the      */
/* Lodscore programs.  This routine sets the parameters of the QTL.           */
/* It replaces schl_set_traitdat () for schnell.                              */

/* This subroutine is written without requiring that all trait loci come in   */
/* front of any marker loci in "loci", the global locus array of structures.  */
/* It will still run no matter where in "loci" the tlocs are located.         */

void set_quant_trait ()
{
   TRAITPARS_PTR p_traitp;
   TLOCPARS_PTR p_tlocp;

   double f1, f2;                       /* Allele frequencies.                */
   double mu;                           /* Mean trait value at current tloc.  */
   int   n_all, n_gen;
   int   i1, i2;

   p_traitp = P_trait_pars;
   p_tlocp = p_traitp->p_tloc_pars;
   fflush (stdout);

   for (i1 = 0; i1 < NLoci; ++i1)
      if (loci[i1].locus_type == TRAIT_LOCUS)
      {
         loci[i1].va = p_traitp->addi_var;
         loci[i1].ve = p_traitp->resi_var;
         printf ("  Residual variance = %f, Additive Variance = %f\n",
                 loci[i1].ve, loci[i1].va);

         n_all = loci[i1].n_all;
         if (n_all != 2)
         {
            fprintf (stderr,
                     "Error: expect 2 alleles for the trait, got %d!\n", n_all);
            fflush (stderr);
            exit (EXIT_FAILURE);
         }

         /* Given three genotypic means, use allele frequencies to compute    */
         /* global means, since there are only two alleles for the trait.     */

         f1 = loci[i1].freq[0];
         f2 = loci[i1].freq[1];
         mu = f1 * f1 * p_tlocp->means[0] + 2.0 * f1 * f2 * p_tlocp->means[1]
            + f2 * f2 * p_tlocp->means[2];
         printf ("  Trait global mean = %f\n", mu);
         printf ("  Trait Id %d: no of alleles = %d\n", i1, n_all);

         n_gen = n_all * (n_all + 1) / 2;
         for (i2 = 0; i2 < n_gen; i2++)
         {
            loci[i1].g_mu[i2] = p_tlocp->means[i2];
            printf ("    Genotype %d, deviation from mean: %f\n", i2,
                    loci[i1].g_mu[i2] - mu);
         }
         ++p_tlocp;
         printf ("\n");
      }

   printf ("\n");
   fflush (stdout);

   return;
}
